/*
 * Copyright 2005 Jean-Philippe Aumasson <jpnossa@free.fr>
 * Copyright 2005 Florent Kaisser <jpnossa@free.fr>
 * Copyright 2005 Jean-Baptiste Langlois <florent.kaisser@free.fr>
 *
 * This file is part of SIMS : SIMS Is a Machine Simulator
 *
 * SIMS is free software; you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation; either version 2 of the License, or
 * (at your option) any later version.
 *
 * SIMS is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with SIMS; if not, write to the Free Software
 * Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
 *
 * For more details see the file COPYING.
 */
/**
* l'interface pour permettre au Graphe de voir la machine  afficher
*
*/
package sims;

import java.util.*;


public interface GrapheInterface {
    /**
    * Main method needed to build the graph, describes the links between vertices.
    *
    * @see Arc.java
    * @return a collection of Arc instances
    */
    public Collection arcs(); //retourne une collection d'Arc

    /**
    * Gives the number of states of the machine, hence the number of vertices in the graph.
    *
    * @return the number of states of the machine.
    */
    public int getNbrEtat();

    /**
    * Gives the number of edges in the graph, equals to the number of instructions of a Turing Machine.
    *
    * @return the number of edges..
    */
    public int getNbrArc();

    /**
    * Gives the root state of a machine, the start vertex in the graph.
    *
    * @return the root state.
    */
    public Object getRacine();
}
