/*
 * Copyright 2005 Jean-Philippe Aumasson <jpnossa@free.fr>
 * Copyright 2005 Florent Kaisser <jpnossa@free.fr>
 * Copyright 2005 Jean-Baptiste Langlois <florent.kaisser@free.fr>
 *
 * This file is part of SIMS : SIMS Is a Machine Simulator
 *
 * SIMS is free software; you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation; either version 2 of the License, or
 * (at your option) any later version.
 *
 * SIMS is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with SIMS; if not, write to the Free Software
 * Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
 *
 * For more details see the file COPYING.
 */
/**
 * Fenetre des commandes de la simulation
 *
 * @author florent.kaisser@free.fr
 * @author JP <jpnossa@free.fr>
 * @version 20/12/04
 *
 */
package sims.ihm;

import sims.*;

import sims.ihm.themes.*;

import java.awt.*;
import java.awt.event.*;

import javax.swing.*;


public class Commandes extends JToolBar implements GraphicValues, BandW {
    private InterfaceSimulation sim;
    private boolean goOn = false;
    private boolean stepOn = false;
    private JButton stopper;
    private JButton nextstepper;
    private JButton speeder;
    private JButton unspeeder;
    private JButton reseter;
    private JLabel tempo;
    private Color buttonActive = Color.RED;

    ////////////////////////////////////////////////////////////////////
    public Commandes(InterfaceSimulation s) {
        super("Commands ");
        sim = s;

        // POSITIONNING
        setSize(COMMANDS_WIDTH, COMMANDS_HEIGHT);

        Dimension dim = Toolkit.getDefaultToolkit().getScreenSize();
        setLocation((dim.width / 2) - (getWidth() / 2),
            (dim.height / 2) - (getHeight() / 2) + (VIEW_HEIGHT / 2) + 50);

        // BUTTONS
        stopper = new JButton(new ImageIcon(CMD_GO));
        stopper.setSize(CMD_BUTTON_WIDTH, CMD_BUTTON_HEIGHT);
        stopper.setMnemonic(KeyEvent.VK_F2);
        stopper.addActionListener(new ActionListener() {
                public void actionPerformed(ActionEvent event) {
                    if (goOn) {
                        sim.arreter();
                        stopper.setIcon(new ImageIcon(CMD_GO));
                        stopper.setToolTipText("Start");
                    } else {
                        sim.demarrer();
                        stopper.setIcon(new ImageIcon(CMD_STOP));
                        stopper.setToolTipText("Stop");
                    }

                    goOn = !goOn;
                }
            });

        reseter = new JButton(new ImageIcon(CMD_RESET));
        reseter.setSize(CMD_BUTTON_WIDTH, CMD_BUTTON_HEIGHT);
        reseter.setMnemonic(KeyEvent.VK_F4);
        reseter.addActionListener(new ActionListener() {
                public void actionPerformed(ActionEvent event) {
                    sim.reinitialiser();
                }
            });

        nextstepper = new JButton(new ImageIcon(CMD_NEXT));
        nextstepper.setMnemonic(KeyEvent.VK_F3);
        nextstepper.setSize(CMD_BUTTON_WIDTH, CMD_BUTTON_HEIGHT);
        nextstepper.addActionListener(new ActionListener() {
                public void actionPerformed(ActionEvent event) {
                    sim.prochainPas();
                }
            });

        speeder = new JButton(new ImageIcon(CMD_SPEED_UP));
        speeder.setMnemonic(KeyEvent.VK_PLUS);
        speeder.setSize(CMD_BUTTON_WIDTH, CMD_BUTTON_HEIGHT);
        speeder.addActionListener(new ActionListener() {
                public void actionPerformed(ActionEvent event) {
                    sim.incVitesse();
                    actualiserVitesse();
                }
            });

        unspeeder = new JButton(new ImageIcon(CMD_SPEED_DOWN));
        unspeeder.setMnemonic(KeyEvent.VK_MINUS);
        unspeeder.setSize(CMD_BUTTON_WIDTH, CMD_BUTTON_HEIGHT);
        unspeeder.addActionListener(new ActionListener() {
                public void actionPerformed(ActionEvent event) {
                    sim.decVitesse();
                    actualiserVitesse();
                }
            });

        stopper.setToolTipText("Start");
        nextstepper.setToolTipText("Next step");
        reseter.setToolTipText("Reset");
        speeder.setToolTipText("Speed up (" + sim.getVitesse() + ")");
        unspeeder.setToolTipText("Speed down (" + sim.getVitesse() + ")");

        add(stopper);
        add(nextstepper);
        add(reseter);

        add(unspeeder);
        add(speeder);

        // pan.add(tempo);
        setVisible(true);
    }

    ////////////////////////////////////////////////////////////////////

    /**
    * Called at the end of simulation, to reset the Button "Go".
    */
    public void resetButtonGo() {
        stopper.setIcon(new ImageIcon(CMD_GO));
        stopper.setToolTipText("Start");
        goOn = false;
    }

    ////////////////////////////////////////////////////////////////////

    /**
    * To enable or disable all the command buttons.
    *
    * @param enable boolean value, if true will enable the buttons.
    */
    public void enableButtons(boolean enable) {
        stopper.setEnabled(enable);
        nextstepper.setEnabled(enable);
        speeder.setEnabled(enable);
        unspeeder.setEnabled(enable);
        reseter.setEnabled(enable);
    }

    ////////////////////////////////////////////////////////////////////

    /**
    * Update the speed control buttons tooltip texts.
    *
    */
    public void actualiserVitesse() {
        speeder.setToolTipText("Speed up (" + sim.getVitesse() + ")");
        unspeeder.setToolTipText("Speed down (" + sim.getVitesse() + ")");
    }
}
