/*
 * Copyright 2005 Jean-Philippe Aumasson <jpnossa@free.fr>
 * Copyright 2005 Florent Kaisser <jpnossa@free.fr>
 * Copyright 2005 Jean-Baptiste Langlois <florent.kaisser@free.fr>
 *
 * This file is part of SIMS : SIMS Is a Machine Simulator
 *
 * SIMS is free software; you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation; either version 2 of the License, or
 * (at your option) any later version.
 *
 * SIMS is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with SIMS; if not, write to the Free Software
 * Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
 *
 * For more details see the file COPYING.
 */
//$Id: Editeur.java 1.2 2004/11/28 21:37:59 Florent Exp Florent $
package sims.ihm;

import java.awt.*;

import java.io.*;

import java.util.*;

import javax.swing.*;
import javax.swing.text.*;


/**
 * JTextPane, avec la possibilit de surligner une ligne
 *
 * @author florent.kaisser@free.fr
 * @version $Id: Editeur.java 1.2 2004/11/28 21:37:59 Florent Exp Florent $
 *
 * $Log: Editeur.java $
 * Revision 1.2  2004/11/28 21:37:59  Florent
 * Modification de setHighliteLine, pour avoir une couleur souhaite
 *
 * Revision 1.1  2004/11/27 15:27:54  Florent
 * Initial revision
 *
 */
public class Editeur extends JTextPane {
    private Vector retour; //Vecteur des position des retour  la ligne dans le texte de l'editeur
    private LinePainter lpJaune; //Classe pour le surlignement d'une ligne et jaune
    private LinePainter lpRouge; //Classe pour le surlignement d'une ligne et rouge

    public Editeur() {
        lpJaune = new LinePainter(this, Color.YELLOW);
        lpRouge = new LinePainter(this, Color.RED);
    }

    /**
     * actualise le tableau des retours  la ligne (vecteur retour)
     *
     */
    public void updateLigne() {
        retour = new Vector();
        retour.add(new Integer(-1));

        String s;

        try {
            //Recupere le texte contenue dans l'editeur
            s = getDocument().getText(0, getDocument().getLength());

            //Pour chaque caractere, regarde si c'est un retour,
            //si oui, ajoute la position au vecteur
            for (int i = 0; i < s.length(); i++)
                if (s.charAt(i) == '\n') {
                    retour.add(new Integer(i));
                }

            //ajoute les dernier caractre, comme un retour  la ligne
            retour.add(new Integer(s.length()));
        } catch (BadLocationException ble) {
        }
    }

    /**
     * surcharge la fonction read, pour actualiser les retour  la ligne
     * @param f File  lire
     * @throws IOException
     */
    public void read(File f) throws IOException {
        read(new FileInputStream(f), (Object) f);
        updateLigne();
    }

    /**
     * surcharge la fonction read, pour actualiser les retour  la ligne
     * @param f File  lire
     * @throws IOException
     */
    public void write(File f) throws IOException {
        write(new FileWriter(f));
        updateLigne();
    }

    /**
     * Surligne une ligne
     * @param ligne N de ligne  surligner
     * @param c Couleur de surlignage : Color.YELLOW ou Color.RED
     */
    public void setLigneSurbrillance(int ligne, Color c) {
        int begin = ((Integer) retour.get(ligne)).intValue() + 1;
        getHighlighter().removeAllHighlights();

        if (c == Color.RED) {
            lpRouge.addHighlightLigne(begin);
        }

        if (c == Color.YELLOW) {
            lpJaune.addHighlightLigne(begin);
        }

        Rectangle r = rectangleLigne(this, begin);

        if (r != null) {
            //Ractualise la partie de la fenetre qui sera modifi
            repaint(r.x, r.y, r.width, r.height);
        }
    }

    /**
     * Supprimer toute les ligne misent en surbrillance
     *
     */
    public void enleverSurbrillance() {
        getHighlighter().removeAllHighlights();
    }

    /**
     * Calcul les coordonnes du rectangle couvrant une ligne
     * @param c
     * @param offs0 Position du premier caractere de la ligne
     * @return
     */
    private Rectangle rectangleLigne(JTextComponent c, int offs0) {
        try {
            //Rectangle convrant le premier caractere de la ligne
            Rectangle r = c.modelToView(offs0);

            if (r != null) {
                r.x = 0;
                r.width = c.getSize().width;
            }

            return r;
        } catch (BadLocationException e) {
            return null;
        }
    }

    public class LinePainter extends DefaultHighlighter.DefaultHighlightPainter {
        private Highlighter highlighter;
        private Object lastHighlight;

        public LinePainter(JTextComponent component, Color color) {
            super(color);
            highlighter = component.getHighlighter();
        }

        /**
         * This method is overridden to get desired behaviour.
         *
         * Paints a portion of a highlight.
         *
         * @param g the graphics context
         * @param offs0 the starting model offset >= 0
         * @param offs1 the ending model offset >= offs1
         * @param bounds the bounding box of the view, which is not
         *                necessarily the region to paint.
         * @param c the editor
         * @param view View painting for
         * @return region drawing occured in
         */
        public Shape paintLayer(Graphics g, int offs0, int offs1, Shape bounds,
            JTextComponent c, View view) {
            Rectangle r = rectangleLigne(c, offs0);

            // Only use the first offset to get the line to highlight
            g.setColor(getColor());

            g.fillRect(r.x, r.y, r.width, r.height);

            return r;
        }

        private void addHighlightLigne(int offset) {
            try {
                highlighter.addHighlight(offset, offset + 1, this);
            } catch (BadLocationException ble) {
            }
        }
    }
}
