/*
 * Copyright 2005 Jean-Philippe Aumasson <jpnossa@free.fr>
 * Copyright 2005 Florent Kaisser <jpnossa@free.fr>
 * Copyright 2005 Jean-Baptiste Langlois <florent.kaisser@free.fr>
 *
 * This file is part of SIMS : SIMS Is a Machine Simulator
 *
 * SIMS is free software; you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation; either version 2 of the License, or
 * (at your option) any later version.
 *
 * SIMS is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with SIMS; if not, write to the Free Software
 * Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
 *
 * For more details see the file COPYING.
 */
/**
*
* @author <jpnossa@free.fr>
* @version ???
*
*/
package sims.ihm;

import java.awt.*;
import java.awt.event.*;
import java.awt.geom.*;

import java.io.*;

import java.net.URL;

import javax.swing.*;
import javax.swing.event.*;


public class HelpWindow extends JFrame implements GraphicValues,
    HyperlinkListener {
    private static final String title = "Help";
    private static final String location = "file:sims/help.html";
    private JEditorPane pane;

    public HelpWindow() {
        super();
        setTitle(title);
        setSize(HELP_WIDTH, HELP_HEIGHT);
        setResizable(false);
        getContentPane().setLayout(new BorderLayout());

        Dimension dim = Toolkit.getDefaultToolkit().getScreenSize();
        setLocation((dim.width / 2) - (getWidth() / 2),
            (dim.height / 2) - (getHeight() / 2));

        WindowListener l = new WindowAdapter() {
                public void windowClosing(WindowEvent e) {
                    setVisible(false);
                    dispose();
                }
            };

        addWindowListener(l);

        JPanel topPanel = new JPanel();
        topPanel.setLayout(new BorderLayout());
        getContentPane().add(topPanel, BorderLayout.CENTER);

        try {
            pane = new JEditorPane(location);
            pane.setEditable(false);

            JScrollPane spane = new JScrollPane();
            spane.getViewport().add(pane, BorderLayout.CENTER);
            topPanel.add(spane, BorderLayout.CENTER);
        } catch (IOException e) {
            System.out.println("error displaying html file");
        }

        pane.addHyperlinkListener(this);

        setVisible(true);
    }

    ////////////////////////////////////////////////////////////////////
    public void hyperlinkUpdate(HyperlinkEvent event) {
        if (event.getEventType() == HyperlinkEvent.EventType.ACTIVATED) {
            try {
                pane.setPage(event.getURL());
            } catch (IOException ioe) {
                System.out.println("Can't follow link to " +
                    event.getURL().toExternalForm() + ": " + ioe);
            }
        }
    }

    ////////////////////////////////////////////////////////////////////
    private void quit() {
        setVisible(false);
        dispose();

        try {
            finalize();
        } catch (Throwable t) {
        }
    }
}
