/*
 * Copyright 2005 Jean-Philippe Aumasson <jpnossa@free.fr>
 * Copyright 2005 Florent Kaisser <jpnossa@free.fr>
 * Copyright 2005 Jean-Baptiste Langlois <florent.kaisser@free.fr>
 *
 * This file is part of SIMS : SIMS Is a Machine Simulator
 *
 * SIMS is free software; you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation; either version 2 of the License, or
 * (at your option) any later version.
 *
 * SIMS is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with SIMS; if not, write to the Free Software
 * Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
 *
 * For more details see the file COPYING.
 */
/**
 * Filter for files opening in the IHM only with "tur" or "reg" extension
 *
*  @author JP <jpnossa@free.fr>
*  @version 20/12/04
 */
package sims.ihm;

import sims.ihm.*;

import sims.mdt.*;

import java.io.File;

import javax.swing.*;
import javax.swing.filechooser.*;


public class MachineFilter extends FileFilter {
    protected static final String tmExtension = "tur";
    protected static final String rmExtension = "reg";
    protected String description;

    ////////////////////////////////////////////////////////////////////
    public MachineFilter() {
        description = "machines (*.tur; *.reg)";
    }

    ////////////////////////////////////////////////////////////////////

    /**
    * Test if the file matches the filter criterias: if its extension is "reg" or "tur".
    *
    * @param f the file passing the filter.
    * @return <tt>true</tt> if the file is accepted by the filter, false otherwise
    */
    public boolean accept(File f) {
        if (f.isDirectory()) {
            return true;
        }

        String extension = getExtension(f);

        if (extension != null) {
            if ((extension.equals(tmExtension)) ||
                    (extension.equals(rmExtension))) {
                return true;
            }
        } else {
            return false;
        }

        return false;
    }

    ////////////////////////////////////////////////////////////////////

    /**
    * Gives the String that will be printed on the dialog box when applying the filter.
    *
    * @return a String describing the filter.
    */
    public String getDescription() {
        return description;
    }

    ////////////////////////////////////////////////////////////////////

    /**
    * Build a substring of the filename corresponding to its extension (the word after the last occurence of '.').
    * @param f the File we want to get the extension.
    * @return f extension
    */
    public static String getExtension(File f) {
        String ext = null;
        String s = f.getName();
        int i = s.lastIndexOf('.');

        if ((i > 0) && (i < (s.length() - 1))) {
            ext = s.substring(i + 1).toLowerCase();

            return ext;
        }

        // no extension
        return "noext";
    }
}
