/*
 * Copyright 2005 Jean-Philippe Aumasson <jpnossa@free.fr>
 * Copyright 2005 Florent Kaisser <jpnossa@free.fr>
 * Copyright 2005 Jean-Baptiste Langlois <florent.kaisser@free.fr>
 *
 * This file is part of SIMS : SIMS Is a Machine Simulator
 *
 * SIMS is free software; you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation; either version 2 of the License, or
 * (at your option) any later version.
 *
 * SIMS is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with SIMS; if not, write to the Free Software
 * Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
 *
 * For more details see the file COPYING.
 */
/**
 * Main window of the application, to open or create a file.
 *
*  @author JP <jpnossa@free.fr>
*  @version 20/12/04
 */
package sims.ihm;

import sims.*;

import sims.ihm.mar.*;

import sims.ihm.mdt.*;

import sims.mdt.*;

import java.awt.*;
import java.awt.event.*;
import java.awt.geom.*;

import java.io.*;

import javax.swing.*;
import javax.swing.event.*;


public class MainWindow extends JFrame implements GraphicValues {
    private static final String machinePath = "sims/examples";
    private static final String title = "Sims: Sims Is a Machine Simulator";

    public MainWindow(String args) {
        if (args.equals("--macdo")) {
            setTitle("SIMS: Sims Is a Macdonald's Simulator");
        } else {
            setTitle(title);
        }

        setSize(MAIN_WIDTH, MAIN_HEIGHT);
        setResizable(false);

        WindowListener l = new WindowAdapter() {
                public void windowClosing(WindowEvent e) {
                    setVisible(false);
                    System.exit(0);
                }
            };

        addWindowListener(l);

        // places the window at the center of screen
        Dimension dim = Toolkit.getDefaultToolkit().getScreenSize();
        setLocation((dim.width / 2) - (getWidth() / 2),
            (dim.height / 2) - (getHeight() / 2));

        LogoPanel alan = new LogoPanel(args);
        getContentPane().add(alan);

        JMenuBar menuBar = new JMenuBar();
        JMenu mFile = new JMenu("Machine");
        mFile.setMnemonic('M');

        JMenuItem miOpen = new JMenuItem("Open");
        miOpen.addActionListener(new ActionListener() {
                public void actionPerformed(ActionEvent event) {
                    JFileChooser fileSelect = new JFileChooser();
                    fileSelect.setCurrentDirectory(new File(machinePath));
                    fileSelect.setMultiSelectionEnabled(false);
                    fileSelect.addChoosableFileFilter(new MachineFilter());
                    fileSelect.setAcceptAllFileFilterUsed(false);

                    int result = fileSelect.showOpenDialog(MainWindow.this);

                    if ((fileSelect.getSelectedFile() != null) &&
                            (result == JFileChooser.APPROVE_OPTION)) { // if we selected a file

                        String fileName = fileSelect.getSelectedFile().getPath();

                        if (TurFilter.getExtension(fileSelect.getSelectedFile())
                                         .equals("tur")) {
                            new TmLauncher(fileName);
                        } else {
                            new Main(fileName);
                        }
                    }
                }
            });

        miOpen.setAccelerator(KeyStroke.getKeyStroke(KeyEvent.VK_O,
                InputEvent.CTRL_MASK));

        JMenu meNew = new JMenu("New");

        JMenuItem newTuring = new JMenuItem("Turing machine");
        JMenuItem newRam = new JMenuItem("Register machine");

        newTuring.addActionListener(new ActionListener() {
                public void actionPerformed(ActionEvent event) {
                    JFileChooser fileSelect = new JFileChooser();
                    fileSelect.setCurrentDirectory(new File(machinePath));
                    fileSelect.setMultiSelectionEnabled(false);
                    fileSelect.addChoosableFileFilter(new TurFilter());
                    fileSelect.setAcceptAllFileFilterUsed(false);

                    int result = fileSelect.showSaveDialog(MainWindow.this);

                    if (fileSelect.getSelectedFile() != null) { // if we saved a file

                        String fileName = fileSelect.getSelectedFile().getPath();

                        if (TurFilter.getExtension(fileSelect.getSelectedFile())
                                         .equals("tur")) {
                            new TmLauncher(fileName);
                        } else {
                            new TmLauncher(fileName + ".tur");
                        }
                    }
                }
            });

        newRam.addActionListener(new ActionListener() {
                public void actionPerformed(ActionEvent event) {
                    JFileChooser fileSelect = new JFileChooser();
                    fileSelect.setCurrentDirectory(new File(machinePath));
                    fileSelect.setMultiSelectionEnabled(false);
                    fileSelect.addChoosableFileFilter(new RegFilter());
                    fileSelect.setAcceptAllFileFilterUsed(false);

                    int result = fileSelect.showSaveDialog(MainWindow.this);

                    if (fileSelect.getSelectedFile() != null) { // if we saved a file

                        String fileName = fileSelect.getSelectedFile().getPath();

                        if (TurFilter.getExtension(fileSelect.getSelectedFile())
                                         .equals("reg")) {
                            /*    LE FICHIER SERA VIDE !    A GERE DANS TM LAUNCHER             */
                            new Main(fileName);
                        }
                    }
                }
            });

        JMenuItem miQuit = new JMenuItem("Quit");
        miQuit.addActionListener(new ActionListener() {
                public void actionPerformed(ActionEvent event) {
                    System.exit(0);
                }
            });

        // shortcuts
        newTuring.setAccelerator(KeyStroke.getKeyStroke(KeyEvent.VK_T,
                InputEvent.CTRL_MASK));

        newRam.setAccelerator(KeyStroke.getKeyStroke(KeyEvent.VK_R,
                InputEvent.CTRL_MASK));

        miOpen.setAccelerator(KeyStroke.getKeyStroke(KeyEvent.VK_O,
                InputEvent.CTRL_MASK));

        miQuit.setAccelerator(KeyStroke.getKeyStroke(KeyEvent.VK_Q,
                InputEvent.CTRL_MASK));

        JMenu help = new JMenu("?");
        help.setMnemonic('?');

        JMenuItem aide = new JMenuItem("Help");
        aide.setAccelerator(KeyStroke.getKeyStroke(KeyEvent.VK_F1, 0));
        aide.addActionListener(new ActionListener() {
                public void actionPerformed(ActionEvent e) {
                    new HelpWindow();
                }
            });

        JMenuItem cred = new JMenuItem("About...");

        cred.addActionListener(new ActionListener() {
                public void actionPerformed(ActionEvent e) {
                    new CreditWindow();
                }
            });

        mFile.setFont(POLICE);
        meNew.setFont(POLICE);
        miOpen.setFont(POLICE);
        newRam.setFont(POLICE);
        newTuring.setFont(POLICE);
        miQuit.setFont(POLICE);
        help.setFont(POLICE);
        aide.setFont(POLICE);
        cred.setFont(POLICE);

        help.add(aide);
        help.add(cred);

        meNew.add(newTuring);
        meNew.add(newRam);
        mFile.add(meNew);
        mFile.add(miOpen);

        mFile.addSeparator();
        mFile.add(miQuit);

        setJMenuBar(menuBar);
        menuBar.add(mFile);

        menuBar.add(help);

        setVisible(true);
    }

    ////////////////////////////////////////////////////////////////////

    /**
    * panel to show the splash image
    */
    class LogoPanel extends JPanel {
        private Image image;

        public LogoPanel(String args) {
            if (args.equals("--macdo")) {
                image = Toolkit.getDefaultToolkit().getImage("sims/ihm/visuals/macdo.jpg");
            } else {
                image = Toolkit.getDefaultToolkit().getImage("sims/ihm/visuals/turingbus.jpg");
            }

            MediaTracker tracker = new MediaTracker(this);
            tracker.addImage(image, 0);

            try {
                tracker.waitForID(0);
            } catch (InterruptedException exception) {
            }
        }

        public void paintComponent(Graphics gr) {
            super.paintComponent(gr);

            Graphics2D g = (Graphics2D) gr;
            g.drawImage(image, 0, 0, null);
            setBackground(Color.WHITE);
        }
    }
}
