package sims.ihm.graph;


/*
 * Copyright 2005 Jean-Philippe Aumasson <jpnossa@free.fr>
 * Copyright 2005 Florent Kaisser <jpnossa@free.fr>
 * Copyright 2005 Jean-Baptiste Langlois <florent.kaisser@free.fr>
 *
 * This file is part of SIMS : SIMS Is a Machine Simulator
 *
 * SIMS is free software; you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation; either version 2 of the License, or
 * (at your option) any later version.
 *
 * SIMS is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with SIMS; if not, write to the Free Software
 * Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
 *
 * For more details see the file COPYING.
 */

/**
*  Class to draw an arrow for an oriented graph.
*  @author Jean-Baptiste Langlois
*  @version 1.0beta1
*  Arrow2D.java
*/
import java.awt.*;
import java.awt.event.*;
import java.awt.geom.*;

import javax.swing.*;


public class Arrow2D {
    private Graphics2D graph;
    private float xArrow;
    private float yArrow;
    private float xfArrow;
    private float yfArrow;
    private float xTriangle;
    private float yTriangle;
    private int direction;
    private int angle;
    private GraphState2D s1;
    private GraphState2D s2;
    private Graph gr;

    /**
    *  Arrow2D Constructor to build an arrow between two states
    *  @param ObjectToDraw
    *  @param SourceState
    *  @param DestinationState
    *  Arrow2D()
    */
    public Arrow2D(Graphics2D g2, GraphState2D state1, GraphState2D state2,
        Graph gr) {
        // Arrow between two states
        graph = g2;
        s2 = state2;
        s1 = state1;
        xArrow = state1.getX();
        xfArrow = state2.getX();
        yArrow = state1.getY();
        yfArrow = state2.getY();
        this.gr = gr;

        //setCoordinates();
        setAngle();
    }

    /**
    *  Arrow2D Constructor to build an arrow on a self-state
    *  @param ObjectToDraw
    *  @param State
    *  @param Position
    *  Arrow2D()
    */
    public Arrow2D(Graphics2D g2, GraphState2D state1, int dir) {
        // dir specify where the 'loop arrow' must be displayed
        xArrow = state1.getX();
        xfArrow = state1.getX();
        yArrow = state1.getY();
        yfArrow = state1.getY();
        direction = dir;
        s1 = state1;
        s2 = state1;
        graph = g2;
        this.gr = null;

        setAngle();
    }

    /**
    *  Arrow2D Constructor to build an arrow between two points
    *  @param ObjectToDraw
    *  @param SourceX
    *  @param SourceY
    *  @param DestinationX
    *  @param DestinationY
    *  Arrow2D()
    */
    public Arrow2D(Graphics2D g2, float SourceX, float SourceY, float DestX,
        float DestY) {
        xArrow = SourceX;
        xfArrow = DestX;
        yArrow = SourceY;
        yfArrow = DestY;
        setAngle();
    }

    /**
    *  Set the coordinates of the arrow and the triangle
    *  setCoordinates()
    */
    public void setCoordinates() {
        xArrow = s1.getX();
        xfArrow = s2.getX();
        yArrow = s1.getY();
        yfArrow = s2.getY();

        // If the X-coordinates and Y-coordinates of the 2 states are different
        if ((getState1().getX() != getState2().getX()) &&
                (getState1().getY() != getState2().getY())) {
            if ((xArrow > xfArrow) && (yArrow > yfArrow)) {
                xTriangle = (float) xfArrow +
                    (float) ((getState2().getRadiusW() / 2) * (float) Math.cos((float) Math.atan(
                            (float) (yArrow - yfArrow) / (float) (xArrow -
                            xfArrow))));
                yTriangle = (float) yfArrow +
                    (float) ((getState2().getRadius() / 2) * (float) Math.sin((float) Math.atan(
                            (float) (yArrow - yfArrow) / (float) (xArrow -
                            xfArrow))));
            }

            if ((xArrow > xfArrow) && (yArrow < yfArrow)) {
                xTriangle = (float) xfArrow +
                    (float) ((getState2().getRadiusW() / 2) * (float) Math.cos((float) Math.atan(
                            (float) (yArrow - yfArrow) / (float) (xArrow -
                            xfArrow))));
                yTriangle = (float) yfArrow +
                    (float) ((getState2().getRadius() / 2) * (float) Math.sin((float) Math.atan(
                            (float) (yArrow - yfArrow) / (float) (xArrow -
                            xfArrow))));
            }

            if ((xArrow < xfArrow) && (yArrow > yfArrow)) {
                xTriangle = (float) xfArrow -
                    (float) ((getState2().getRadiusW() / 2) * (float) Math.cos((float) Math.atan(
                            (float) (yArrow - yfArrow) / (float) (xArrow -
                            xfArrow))));
                yTriangle = (float) yfArrow -
                    (float) ((getState2().getRadius() / 2) * (float) Math.sin((float) Math.atan(
                            (float) (yArrow - yfArrow) / (float) (xArrow -
                            xfArrow))));
            }

            if ((xArrow < xfArrow) && (yArrow < yfArrow)) {
                xTriangle = (float) xfArrow -
                    (float) ((getState2().getRadiusW() / 2) * (float) Math.cos((float) Math.atan(
                            (float) (yArrow - yfArrow) / (float) (xArrow -
                            xfArrow))));
                yTriangle = (float) yfArrow -
                    (float) ((getState2().getRadius() / 2) * (float) Math.sin((float) Math.atan(
                            (float) (yArrow - yfArrow) / (float) (xArrow -
                            xfArrow))));
            }
        }

        if (gr != null) {
            if (gr.isSucc(s2, s1)) {
                if ((xArrow > xfArrow) && (yArrow != yfArrow)) {
                    xArrow += ((getState2().getRadiusW() / 2) * Math.sin(Math.atan(
                            (float) (yArrow - yfArrow) / (float) (xArrow -
                            xfArrow))));

                    //xfArrow += (getState2().getRadius()/4)*Math.sin(Math.atan((float)(yArrow - yfArrow)/(float) (xArrow -xfArrow)));
                    yArrow += ((getState2().getRadiusW() / 2) * Math.cos(Math.atan(
                            (float) (yArrow - yfArrow) / (float) (xArrow -
                            xfArrow))));

                    //yfArrow += (getState2().getRadius()/4)*Math.cos(Math.atan((float)(yArrow - yfArrow)/(float) (xArrow -xfArrow)));
                    //if (yArrow < yfArrow)
                    //{
                    //xTriangle -= (getState2().getRadiusW() / 2)*Math.sin(Math.atan((float)(yfArrow - yArrow)/(float) (xfArrow -xArrow)));
                    //yTriangle -= (getState2().getRadius() / 2)*Math.cos(Math.atan((float)(yfArrow - yArrow)/(float) (xfArrow -xArrow)));
                    xTriangle = (float) xfArrow +
                        (float) ((getState2().getRadiusW() / 2) * (float) Math.cos((float) Math.atan(
                                (float) (yArrow - yfArrow) / (float) (xArrow -
                                xfArrow))));
                    yTriangle = (float) yfArrow +
                        (float) ((getState2().getRadius() / 2) * (float) Math.sin((float) Math.atan(
                                (float) (yArrow - yfArrow) / (float) (xArrow -
                                xfArrow))));

                    //}
                }

                if ((xArrow < xfArrow) && (yArrow != yfArrow)) {
                    xArrow -= ((getState2().getRadiusW() / 2) * Math.sin(Math.atan(
                            (float) (yArrow - yfArrow) / (float) (xArrow -
                            xfArrow))));

                    //xfArrow -= (getState2().getRadius()/4)*Math.sin(Math.atan((float)(yArrow - yfArrow)/(float) (xArrow -xfArrow)));
                    yArrow -= ((getState2().getRadiusW() / 2) * Math.cos(Math.atan(
                            (float) (yArrow - yfArrow) / (float) (xArrow -
                            xfArrow))));

                    //yfArrow -= (getState2().getRadius()/4)*Math.cos(Math.atan((float)(yArrow - yfArrow)/(float) (xArrow -xfArrow)));
                    //if (yArrow > yfArrow)
                    //{
                    //xTriangle -= (getState2().getRadiusW() / 2)*Math.sin(Math.atan((float)(yfArrow - yArrow)/(float) (xfArrow -xArrow)));
                    //yTriangle -= (getState2().getRadius() / 2)*Math.cos(Math.atan((float)(yfArrow - yArrow)/(float) (xfArrow -xArrow)));
                    xTriangle = (float) xfArrow -
                        (float) ((getState2().getRadiusW() / 2) * (float) Math.cos((float) Math.atan(
                                (float) (yArrow - yfArrow) / (float) (xArrow -
                                xfArrow))));
                    yTriangle = (float) yfArrow -
                        (float) ((getState2().getRadius() / 2) * (float) Math.sin((float) Math.atan(
                                (float) (yArrow - yfArrow) / (float) (xArrow -
                                xfArrow))));

                    //}
                }
            }
        }

        if ((xArrow == xfArrow) && (yArrow != yfArrow)) {
            xTriangle = (float) xArrow;

            if (yArrow > yfArrow) {
                yTriangle = (float) getState2().getY() +
                    (getState2().getRadius() / 2);
            } else {
                yTriangle = (float) getState2().getY() -
                    (getState2().getRadius() / 2);
            }
        }

        // If the Y-coordinates of the 2 states are identical 
        if ((getState1().getX() != getState2().getX()) &&
                (getState1().getY() == getState2().getY())) {
            if (getState1().getX() < getState2().getX()) {
                xTriangle = (float) getState2().getX() -
                    (getState2().getRadiusW() / 2);
            } else {
                xTriangle = (float) getState2().getX() +
                    (getState2().getRadiusW() / 2);
            }

            yTriangle = (float) getState2().getY();
        }

        if (gr != null) {
            if (gr.isSucc(s2, s1)) {
                if ((xArrow != xfArrow) && (yArrow == yfArrow)) {
                    if (xArrow > xfArrow) {
                        yArrow -= (getState2().getRadius() / 5);
                        yfArrow -= (getState2().getRadius() / 5);
                        yTriangle -= (getState2().getRadius() / 5);
                    }

                    if (xArrow < xfArrow) {
                        yArrow += (getState2().getRadius() / 5);
                        yfArrow += (getState2().getRadius() / 5);
                        yTriangle += (getState2().getRadius() / 5);
                    }
                }
            }
        }

        // If the next vertex is itself
        // We must define the direction according to the place
        // where it must be displayed
        if ((xArrow == xfArrow) && (yArrow == yfArrow)) {
            if (direction == 2) {
                xTriangle = xArrow - (getState2().getRadius() / 4);
                yTriangle = yArrow - (getState2().getRadiusW() / 2);
            }

            if (direction == 3) {
                xTriangle = xArrow - (getState2().getRadius() / 4);
                yTriangle = yArrow + (getState2().getRadiusW() / 2);
            }

            if (direction == 0) {
                xTriangle = xArrow + (getState2().getRadius() / 4);
                yTriangle = yArrow - (getState2().getRadiusW() / 2);
            }

            if (direction == 1) {
                xTriangle = xArrow + (getState2().getRadius() / 2);
                yTriangle = yArrow;
            }
        }

        setAngle();
    }

    /**
    *  Calculation of the angle of the triangular part of the arrow.
    *  setAngle()
    */
    public void setAngle() {
        if ((xArrow == xfArrow) && (yArrow != yfArrow)) {
            if (yfArrow > yArrow) {
                angle = (int) Math.toDegrees(Math.asin(direction * 0.2F)) +
                    270;
            } else {
                angle = (int) Math.toDegrees(Math.asin((-1) * direction * 0.2F)) +
                    90;
            }
        }

        if ((xArrow != xfArrow) && (yArrow == yfArrow)) {
            if (xfArrow > xArrow) {
                angle = (int) Math.toDegrees(Math.acos(direction * 0.2F)) + 90;
            } else {
                angle = (int) Math.toDegrees(Math.acos((-1) * direction * 0.2F)) +
                    270;
            }
        }

        if ((xArrow != xfArrow) && (yArrow != yfArrow)) {
            angle = (int) Math.toDegrees(Math.atan(
                        (yfArrow - yArrow) / (xfArrow - xArrow)));

            if (xArrow < xfArrow) {
                angle += 180;
            }
        }

        if ((xArrow == xfArrow) && (yArrow == yfArrow)) {
            if (direction == 2) {
                angle = 235;
            }

            if (direction == 1) {
                angle = 0;
            }

            if (direction == 0) {
                angle = 300;
            }

            if (direction == 3) {
                angle = 130;
            }
        }
    }

    /**
    *  Accessor to get the X-coordinate of the initial point
    *  @return the X-coordinate of the base of the arrow
    *  getInitialX()
    */
    public int getInitialX() {
        return (int) xArrow;
    }

    /**
    *  Accessor to get the Y-coordinate of the initial point
    *  @return the Y-coordinate of the base of the arrow
    *  getInitialY()
    */
    public int getInitialY() {
        return (int) yArrow;
    }

    /**
    *  Accessor to get the X-coordinate of the final point
    *  @return the X-coordinate of the top of the arrow
    *  getFinalX()
    */
    public int getFinalX() {
        return (int) xfArrow;
    }

    /**
    *  Accessor to get the Y-coordinate of the final point
    *  @return the Y-coordinate of the top of the arrow
    *  getFinalY()
    */
    public int getFinalY() {
        return (int) yfArrow;
    }

    /**
    *  Accessor to get the initial vertex
    *  @return the initial vertex
    *  getState1()
    */
    public GraphState2D getState1() {
        return s1;
    }

    /**
    *  Accessor to get the final vertex
    *  @return the final vertex
    *  getState2()
    */
    public GraphState2D getState2() {
        return s2;
    }

    /**
    *  Accessor to get the value of the angle of the arrow
    *  @return the value of the angle of the arrow
    *  getAngle()
    */
    public int getAngle() {
        return angle;
    }

    /**
    *  Accessor to get the direction for self-vertex edges
    *  @return the direction
    *  getDirection()
    */
    public int getDirection() {
        return direction;
    }

    /**
    *  Accessor to get the direction for self-vertex edges
    *  @param direction
    *  setDirection()
    */
    public void setDirection(int dir) {
        direction = dir;
    }

    /**
    *  Arrow2D method to draw the arrow
    *  Draw()
    */
    public void draw() {
        // Draw the line, heading from the first point to the second
        if ((xArrow == xfArrow) && (yArrow != yfArrow)) {
            graph.draw(new QuadCurve2D.Double((double) xArrow, (double) yArrow,
                    (double) (xArrow +
                    ((direction * Math.abs(yArrow - yfArrow)) / 5)),
                    (yArrow + yfArrow) / 2, (double) xfArrow, (double) yfArrow));
        }

        if ((xArrow != xfArrow) && (yArrow == yfArrow)) {
            graph.draw(new QuadCurve2D.Double((double) xArrow, (double) yArrow,
                    (double) (xArrow + xfArrow) / 2,
                    (double) (yArrow +
                    ((direction * Math.abs(xArrow - xfArrow)) / 5)),
                    (double) xfArrow, (double) yfArrow));
        }

        if ((xArrow != xfArrow) && (yArrow != yfArrow)) {
            graph.draw(new Line2D.Double((double) xArrow, (double) yArrow,
                    (double) xfArrow, (double) yfArrow));
        }

        if ((xArrow == xfArrow) && (yArrow == yfArrow)) {
            if (direction == 0) {
                graph.draw(new Ellipse2D.Double((double) xArrow +
                        (getState1().getRadius() / 4),
                        (double) yArrow -
                        ((getState1().getRadiusW() * 1.5) / 2),
                        getState1().getRadiusW() * 0.75,
                        getState1().getRadiusW() * 0.75));
            }

            if (direction == 2) {
                graph.draw(new Ellipse2D.Double((double) xArrow -
                        getState1().getRadius(),
                        (double) yArrow -
                        ((getState1().getRadiusW() * 1.5) / 2),
                        getState1().getRadiusW() * 0.75,
                        getState1().getRadiusW() * 0.75));
            }

            if (direction == 1) {
                graph.draw(new Ellipse2D.Double((double) xArrow +
                        (getState1().getRadius() / 4), (double) yArrow,
                        getState1().getRadiusW() * 0.75,
                        getState2().getRadiusW() * 0.75));
            }

            if (direction == 3) {
                graph.draw(new Ellipse2D.Double((double) xArrow -
                        getState1().getRadius(), (double) yArrow,
                        getState1().getRadiusW() * 0.75,
                        getState1().getRadiusW() * 0.75));
            }
        }

        // Draw a triangular shape at the end of the curve to simulate an arrow
        int decalage = 30;
        graph.draw(new Line2D.Double((double) xTriangle, (double) yTriangle,
                (double) xTriangle +
                ((getState1().getFontSize() - 4) * Math.cos(Math.toRadians(angle -
                        decalage))),
                (double) yTriangle +
                ((getState1().getFontSize() - 4) * Math.sin(Math.toRadians(angle -
                        decalage)))));
        graph.draw(new Line2D.Double((double) xTriangle, (double) yTriangle,
                (double) xTriangle +
                ((getState1().getFontSize() - 4) * Math.cos(Math.toRadians(angle +
                        decalage))),
                (double) yTriangle +
                ((getState1().getFontSize() - 4) * Math.sin(Math.toRadians(angle +
                        decalage)))));
    }
}
