package sims.ihm.graph;


/*
 * Copyright 2005 Jean-Philippe Aumasson <jpnossa@free.fr>
 * Copyright 2005 Florent Kaisser <jpnossa@free.fr>
 * Copyright 2005 Jean-Baptiste Langlois <florent.kaisser@free.fr>
 *
 * This file is part of SIMS : SIMS Is a Machine Simulator
 *
 * SIMS is free software; you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation; either version 2 of the License, or
 * (at your option) any later version.
 *
 * SIMS is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with SIMS; if not, write to the Free Software
 * Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
 *
 * For more details see the file COPYING.
 */
import sims.*;

import java.awt.*;
import java.awt.event.*;
import java.awt.geom.*;

import java.util.*;

import javax.swing.*;
import javax.swing.event.*;


public class Graph {
    private int GWIDTH = 600;
    private int GHEIGHT = 600;
    private GraphState2D[] states;
    private Graphics2D graph;
    public int nbStates;
    private Transition2D[][] tab;
    private int rootGraph;
    private Vector etats;
    private Arrow2D[] fleche;
    private int maxArrow;

    public Graph(boolean[][] m, int N) {
        nbStates = N;
        states = new GraphState2D[N];

        for (int i = 0; i < N; i++)
            states[i] = new GraphState2D(i);

        for (int i = 0; i < N; i++)
            for (int j = 0; j < N; j++)
                if (m[i][j] == true) {
                    if (i != j) {
                        states[i].addSucc(j);
                        states[j].addPred(i);
                    } else {
                        states[i].isSelfSucc(true);
                    }
                }
    }

    public Graph(Graphics2D g2, boolean[][] m, int N) {
        nbStates = N;
        states = new GraphState2D[N];

        for (int i = 0; i < N; i++)
            states[i] = new GraphState2D(i);

        for (int i = 0; i < N; i++)
            for (int j = 0; j < N; j++)
                if (m[i][j] == true) {
                    if (i != j) {
                        states[i].addSucc(j);
                        states[j].addPred(i);
                    } else {
                        states[i].isSelfSucc(true);
                    }
                }

        graphPlace(g2);
    }

    public Graph(Graphics2D g2, GrapheInterface gr, int N, int width, int height) {
        nbStates = N;
        states = new GraphState2D[N];
        tab = new Transition2D[N][N];
        graph = g2;

        boolean[][] m = new boolean[N][N];
        String[][] t = new String[N][N];
        Object a;
        Object b;

        GWIDTH = width;
        GHEIGHT = height;
        fleche = new Arrow2D[2 * nbStates * nbStates];

        Object[] trans = gr.arcs().toArray();

        etats = new Vector();

        for (int i = 0; i < trans.length; i++) {
            if (!etats.contains(((Arc) trans[i]).getStartVert())) {
                etats.add(((Arc) trans[i]).getStartVert());
            }

            if (!etats.contains(((Arc) trans[i]).getEndVert())) {
                etats.add(((Arc) trans[i]).getEndVert());
            }
        }

        rootGraph = etats.indexOf(gr.getRacine());

        //rootGraph=N-1;
        for (int i = 0; i < N; i++)
            for (int j = 0; j < N; j++) {
                m[i][j] = false;
                t[i][j] = "";
            }

        for (int i = 0; i < trans.length; i++) {
            a = ((Arc) trans[i]).getStartVert();
            b = ((Arc) trans[i]).getEndVert();
            m[etats.indexOf(a)][etats.indexOf(b)] = true;
            t[etats.indexOf(a)][etats.indexOf(b)] = ((Arc) trans[i]).toString();
        }

        int taille;

        if (width > height) {
            taille = height;
        } else {
            taille = width;
        }

        for (int i = 0; i < N; i++)
            states[i] = new GraphState2D(i, etats.get(i).toString(), taille);

        for (int i = 0; i < N; i++)
            for (int j = 0; j < N; j++)
                if (m[i][j] == true) {
                    if (i != j) {
                        states[i].addSucc(j);
                        states[j].addPred(i);
                    } else {
                        states[i].isSelfSucc(true);
                    }
                }

        maxArrow = 0;

        for (int i = 0; i < nbStates; i++)
            for (int j = 0; j < nbStates; j++) {
                if (states[i].isSucc(j)) {
                    fleche[maxArrow] = new Arrow2D(g2, states[i], states[j],
                            this);
                    maxArrow++;
                } else if ((states[i].getSelfSucc()) && (i == j)) {
                    fleche[maxArrow] = new Arrow2D(g2, states[i],
                            getDirection(i));
                    maxArrow++;
                }
            }

        for (int k = 0; k < maxArrow; k++)
            for (int i = 0; i < N; i++)
                for (int j = 0; j < N; j++) {
                    //System.out.println("k : "+k+", maxArrow : "+maxArrow+", i :"+i+", j : "+j);
                    //System.out.println("getState1 : "+fleche[k].getState1().getValue());
                    if ((fleche[k].getState1() == states[i]) &&
                            (fleche[k].getState2() == states[j])) {
                        tab[i][j] = new Transition2D(this, fleche[k], t[i][j],
                                taille);
                    }
                }

        graphPlace(g2);
    }

    public GraphState2D getState(int i) {
        return states[i];
    }

    public boolean isSucc(GraphState2D s1, GraphState2D s2) {
        for (int i = 0; i < nbStates; i++)
            if (s2 == getState(i)) {
                return s1.isSucc(i);
            }

        return false;
    }

    public void graphPlace(Graphics2D g2) {
        for (int i = 0; i < nbStates; i++)
            states[i].GraphPlace(g2);

        graph = g2;
    }

    public Graphics2D getGraphics() {
        return graph;
    }

    public void draw(int i) {
        states[i].draw();
    }

    public void draw() {
        for (int k = 0; k < maxArrow; k++)

            //System.out.println("S1 : "+fleche[k].getState1().getValue()+", S2 : "+fleche[k].getState2().getValue());
            //System.out.println("R : "+fleche[k].getState1().getRadius()+", RW : "+fleche[k].getState1().getRadiusW());
            //System.out.println("R : "+fleche[k].getState2().getRadius()+", RW : "+fleche[k].getState2().getRadiusW());
            fleche[k].draw();

        /*   for (int i = 0; i < nbStates; i++)
               for (int j = 0; j < nbStates; j++) {
                   if (states[i].isSucc(j)) {
                       fleche[k] = new Arrow2D(graph, states[i], states[j]);
                       fleche[k].Draw();
                       k++;
                   } else if ((states[i].getSelfSucc()) && (i == j)) {
                       fleche[k] = new Arrow2D(graph, states[i], getDirection(i));
                       fleche[k].Draw();
                       k++;
                   }
               }*/
        for (int i = 0; i < nbStates; i++)
            states[i].draw();

        for (int t1 = 0; t1 < nbStates; t1++)
            for (int t2 = 0; t2 < nbStates; t2++)
                if (tab[t1][t2] != null) {
                    tab[t1][t2].draw();
                }
    }

    public int getDirection(int i) {
        int PHaut = 0;
        int PGauche = 0;

        for (int m = 0; m < states[i].nbAdjacent(); m++) {
            if (states[states[i].getAdjacent(m)].getY() > states[i].getY()) {
                PHaut--;
            }

            if (states[states[i].getAdjacent(m)].getY() < states[i].getY()) {
                PHaut++;
            }

            if (states[states[i].getAdjacent(m)].getX() < states[i].getX()) {
                PGauche--;
            }

            if (states[states[i].getAdjacent(m)].getX() > states[i].getX()) {
                PGauche++;
            }
        }

        //for(int m=0;m<states[1].nbAdjacent();m++) System.out.print("> "+states[states[1].getAdjacent(m)].getY()+", ");
        int direction;

        if (PHaut < 0) {
            direction = 0;
        } else {
            direction = 1;
        }

        if (PGauche < 0) {
            direction += 0;
        } else {
            direction += 2;
        }

        return direction;
    }

    public void currentState(int current) {
        for (int i = 0; i < nbStates; i++)
            states[i].setCurrent(false);

        states[current].setCurrent(true);
    }

    public void currentState(Object o) {
        if (etats.contains(o)) {
            int current = etats.indexOf(o);

            //System.out.println("etats: " + o.toString() + " index " + current);
            for (int i = 0; i < nbStates; i++)
                states[i].setCurrent(false);

            states[current].setCurrent(true);
        }
    }

    public void correction() {
        boolean tourne = true;

        while (tourne) {
            tourne = false;

            for (int i = 0; i < nbStates; i++)
                for (int j = 0; j < nbStates; j++)
                    if ((i != j) &&
                            (states[i].getRank() == states[j].getRank()) &&
                            (states[i].getOrder() == states[j].getOrder())) {
                        tourne = true;
                        states[i].setRank(states[i].getRank() - 1);

                        int u = crossing(i);
                        states[i].setRank(states[i].getRank() + 2);

                        if (u < crossing(i)) {
                            states[i].setRank(states[i].getRank() - 2);
                        }
                    }
        }
    }

    public void Placement() {
        Rank r = new Rank(this);
        r.initRanks(rootGraph);

        Order o = new Order(this, r);

        //o.initOrder(0, 1);
        o.initOrder(rootGraph, 1);
        correction();
        r.normalize();
        r.normalizeByZero();
        transpose();

        /*
                for (int j = 0; j < nbStates; j++)
                    for (int i = 0; i < nbStates; i++)
                        moveState(i);

                transpose();
                */
        o.normalize();
    }

    public boolean acrossState(int i) {
        for (int j = 0; j < nbStates; j++)
            for (int k = 0; k < getState(j).nbAdjacent(); k++) {
                float m1 = m(j, getState(j).getAdjacent(k));
                float p1 = p(m1, j);

                if ((getState(j).getAdjacent(k) != i) && (i != j) &&
                        (((m1 * getState(i).getOrder()) + p1) == getState(i)
                                                                         .getRank())) {
                    return true;
                }
            }

        return false;
    }

    public void moveState(int j) {
        boolean mieux = false;

        for (int i = 0; ((i < nbStates) && (!mieux)); i++)
            for (int h = 0; ((h < nbStates) && (!mieux)); h++)
                if ((h != i) &&
                        (getState(i).getRank() == getState(j).getRank()) &&
                        (getState(j).isAdjacent(i)) &&
                        (getState(h).getRank() == getState(j).getRank()) &&
                        (getState(j).isAdjacent(h))) {
                    mieux = true;

                    float k = getState(i).getRank();
                    getState(i).setRank(k - 1);

                    int u = crossing(i);
                    getState(i).setRank(k + 1);

                    if ((u < crossing(i)) && (acrossState(i))) {
                        getState(i).setRank(k - 1);
                    }
                }
    }

    public void transpose() {
        for (int i = 0; i < nbStates; i++)
            for (int j = i + 1; j < nbStates; j++) {
                if (crossing(i, j) < crossing(j, i)) {
                    float z = states[i].getRank();
                    states[i].setRank(states[j].getRank());
                    states[j].setRank(z);

                    float y = states[i].getOrder();
                    states[i].setOrder(states[j].getOrder());
                    states[j].setOrder(y);
                }
            }
    }

    public void setCoordinates() {
        Rank r = new Rank(this);
        Order o = new Order(this, r);

        for (int i = 0; i < nbStates; i++) {
            states[i].setX(states[i].getRadiusW() +
                ((states[i].getOrder() * (GWIDTH -
                (2 * states[i].getRadiusW()))) / o.maximumOrder()));

            //System.out.println("i : "+i+", valeur :"+ (states[i].getRadiusW()+((states[i].getOrder() * (GWIDTH-2*states[i].getRadiusW())) / o.maximumOrder())));
            states[i].setY(states[i].getRadius() +
                ((states[i].getRank() * (GHEIGHT - (2 * states[i].getRadius()))) / (r.maximumRank())));
        }

        int maxfl = 0;

        for (int i = 0; i < nbStates; i++)
            for (int j = 0; j < nbStates; j++)
                if (states[i].isSucc(j)) {
                    maxfl++;
                } else if ((states[i].getSelfSucc()) && (i == j)) {
                    fleche[maxfl].setDirection(getDirection(i));
                    maxfl++;
                }

        for (int k = 0; k < maxArrow; k++)
            fleche[k].setCoordinates();

        for (int t1 = 0; t1 < nbStates; t1++)
            for (int t2 = 0; t2 < nbStates; t2++) {
                if ((states[t1].isSucc(t2)) ||
                        ((states[t1].getSelfSucc()) && (t1 == t2))) {
                    tab[t1][t2].calculate();
                }
            }
    }

    public int crossing(int v) {
        int q = 0;

        for (int i = 0; i < nbStates; i++)
            if ((getState(v).isAdjacent(i)) && (i != v)) {
                float m1 = m(v, i);
                float p1 = p(m1, i);

                for (int w = 0; w < nbStates; w++)
                    for (int j = 0; j < nbStates; j++)
                        if ((getState(w).isAdjacent(j)) && (j != w)) {
                            float m2 = m(w, j);
                            float p2 = p(m2, j);
                            float x = ((p1 - p2) / (m2 - m1));
                            float y = (x * m1) + p1;

                            if (BetweenState(i, j, v, w, x, y)) {
                                q++;
                            }
                        }
            }

        return q;
    }

    public int crossing() {
        int q = 0;

        for (int v = 0; v < nbStates; v++)
            for (int i = 0; i < nbStates; i++)
                if ((getState(v).isAdjacent(i)) && (i != v)) {
                    float m1 = m(v, i);
                    float p1 = p(m1, i);

                    for (int w = 0; w < nbStates; w++)
                        for (int j = 0; j < nbStates; j++)
                            if ((getState(w).isAdjacent(j)) && (j != w) &&
                                    (j != i) && (j != v) && (w != v) &&
                                    (i != w)) {
                                float m2 = m(w, j);
                                float p2 = p(m2, j);
                                float x = ((p1 - p2) / (m2 - m1));
                                float y = (x * m1) + p1;

                                if (BetweenState(i, j, v, w, x, y)) {
                                    q++;
                                }
                            }
                }

        return q;
    }

    public int crossing(int v, int w) {
        float z;
        float h;

        if (v < w) {
            z = states[v].getOrder();
            states[v].setOrder(states[w].getOrder());
            states[w].setOrder(z);
            h = states[v].getRank();
            states[v].setRank(states[w].getRank());
            states[w].setRank(h);
        }

        int q = 0;

        for (int i = 0; i < nbStates; i++)
            if ((getState(v).isAdjacent(i)) && (i != v)) {
                float m1 = m(v, i);
                float p1 = p(m1, i);

                for (int j = 0; j < nbStates; j++)
                    if ((getState(w).isAdjacent(j)) && (j != w) && (j != i) &&
                            (j != v) && (w != v) && (i != w)) {
                        float m2 = m(w, j);
                        float p2 = p(m2, j);
                        float x = ((p1 - p2) / (m2 - m1));
                        float y = (x * m1) + p1;

                        if (BetweenState(i, j, v, w, x, y)) {
                            q++;
                        }
                    }
            }

        if (v < w) {
            z = states[v].getOrder();
            states[v].setOrder(states[w].getOrder());
            states[w].setOrder(z);
            h = states[v].getRank();
            states[v].setRank(states[w].getRank());
            states[w].setRank(h);
        }

        return q;
    }

    public float m(int i, int j) {
        return ((states[i].getRank() - states[j].getRank()) / (states[i].getOrder() -
        states[j].getOrder()));
    }

    public float p(float m, int i) {
        return (states[i].getRank() - (m * states[i].getOrder()));
    }

    public boolean BetweenState(int i, int j, int v, int w, float x, float y) {
        if ((getState(i).getOrder() == x) && (getState(i).getRank() == y)) {
            return false;
        }

        if ((getState(j).getOrder() == x) && (getState(j).getRank() == y)) {
            return false;
        }

        if ((getState(v).getOrder() == x) && (getState(v).getRank() == y)) {
            return false;
        }

        if ((getState(w).getOrder() == x) && (getState(w).getRank() == y)) {
            return false;
        }

        if ((getState(i).getOrder() <= x) && (x <= getState(v).getOrder()) &&
                (getState(j).getOrder() <= x) && (x <= getState(w).getOrder()) &&
                (getState(i).getRank() <= y) && (y <= getState(v).getRank()) &&
                (getState(j).getRank() <= y) && (y <= getState(w).getRank())) {
            return true;
        }

        if ((getState(i).getOrder() <= x) && (x <= getState(v).getOrder()) &&
                (getState(j).getOrder() <= x) && (x <= getState(w).getOrder()) &&
                (getState(i).getRank() <= y) && (y <= getState(v).getRank()) &&
                (getState(j).getRank() >= y) && (y >= getState(w).getRank())) {
            return true;
        }

        if ((getState(i).getOrder() <= x) && (x <= getState(v).getOrder()) &&
                (getState(j).getOrder() <= x) && (x <= getState(w).getOrder()) &&
                (getState(i).getRank() >= y) && (y >= getState(v).getRank()) &&
                (getState(j).getRank() >= y) && (y >= getState(w).getRank())) {
            return true;
        }

        if ((getState(i).getOrder() <= x) && (x <= getState(v).getOrder()) &&
                (getState(j).getOrder() >= x) && (x >= getState(w).getOrder()) &&
                (getState(i).getRank() >= y) && (y >= getState(v).getRank()) &&
                (getState(j).getRank() >= y) && (y >= getState(w).getRank())) {
            return true;
        }

        if ((getState(i).getOrder() >= x) && (x >= getState(v).getOrder()) &&
                (getState(j).getOrder() >= x) && (x >= getState(w).getOrder()) &&
                (getState(i).getRank() >= y) && (y >= getState(v).getRank()) &&
                (getState(j).getRank() >= y) && (y >= getState(w).getRank())) {
            return true;
        }

        if ((getState(i).getOrder() <= x) && (x <= getState(v).getOrder()) &&
                (getState(j).getOrder() <= x) && (x <= getState(w).getOrder()) &&
                (getState(i).getRank() >= y) && (y >= getState(v).getRank()) &&
                (getState(j).getRank() <= y) && (y <= getState(w).getRank())) {
            return true;
        }

        if ((getState(i).getOrder() >= x) && (x >= getState(v).getOrder()) &&
                (getState(j).getOrder() >= x) && (x >= getState(w).getOrder()) &&
                (getState(i).getRank() <= y) && (y <= getState(v).getRank()) &&
                (getState(j).getRank() >= y) && (y >= getState(w).getRank())) {
            return true;
        }

        if ((getState(i).getOrder() >= x) && (x >= getState(v).getOrder()) &&
                (getState(j).getOrder() >= x) && (x >= getState(w).getOrder()) &&
                (getState(i).getRank() >= y) && (y >= getState(v).getRank()) &&
                (getState(j).getRank() <= y) && (y <= getState(w).getRank())) {
            return true;
        }

        if ((getState(i).getOrder() <= x) && (x <= getState(v).getOrder()) &&
                (getState(j).getOrder() >= x) && (x >= getState(w).getOrder()) &&
                (getState(i).getRank() <= y) && (y <= getState(v).getRank()) &&
                (getState(j).getRank() >= y) && (y >= getState(w).getRank())) {
            return true;
        }

        if ((getState(i).getOrder() <= x) && (x <= getState(v).getOrder()) &&
                (getState(j).getOrder() >= x) && (x >= getState(w).getOrder()) &&
                (getState(i).getRank() <= y) && (y <= getState(v).getRank()) &&
                (getState(j).getRank() <= y) && (y <= getState(w).getRank())) {
            return true;
        }

        if ((getState(i).getOrder() >= x) && (x >= getState(v).getOrder()) &&
                (getState(j).getOrder() >= x) && (x >= getState(w).getOrder()) &&
                (getState(i).getRank() <= y) && (y <= getState(v).getRank()) &&
                (getState(j).getRank() <= y) && (y <= getState(w).getRank())) {
            return true;
        }

        if ((getState(i).getOrder() >= x) && (x >= getState(v).getOrder()) &&
                (getState(j).getOrder() <= x) && (x <= getState(w).getOrder()) &&
                (getState(i).getRank() <= y) && (y <= getState(v).getRank()) &&
                (getState(j).getRank() <= y) && (y <= getState(w).getRank())) {
            return true;
        }

        if ((getState(i).getOrder() >= x) && (x >= getState(v).getOrder()) &&
                (getState(j).getOrder() <= x) && (x <= getState(w).getOrder()) &&
                (getState(i).getRank() <= y) && (y <= getState(v).getRank()) &&
                (getState(j).getRank() >= y) && (y >= getState(w).getRank())) {
            return true;
        }

        if ((getState(i).getOrder() >= x) && (x >= getState(v).getOrder()) &&
                (getState(j).getOrder() <= x) && (x <= getState(w).getOrder()) &&
                (getState(i).getRank() >= y) && (y >= getState(v).getRank()) &&
                (getState(j).getRank() >= y) && (y >= getState(w).getRank())) {
            return true;
        }

        if ((getState(i).getOrder() <= x) && (x <= getState(v).getOrder()) &&
                (getState(j).getOrder() >= x) && (x >= getState(w).getOrder()) &&
                (getState(i).getRank() >= y) && (y >= getState(v).getRank()) &&
                (getState(j).getRank() <= y) && (y <= getState(w).getRank())) {
            return true;
        }

        if ((getState(i).getOrder() >= x) && (x >= getState(v).getOrder()) &&
                (getState(j).getOrder() <= x) && (x <= getState(w).getOrder()) &&
                (getState(i).getRank() >= y) && (y >= getState(v).getRank()) &&
                (getState(j).getRank() <= y) && (y <= getState(w).getRank())) {
            return true;
        }

        return false;
    }

    ////////////////////////////////////////////////////////////////////
}
