package sims.ihm.graph;


/*
 * Copyright 2005 Jean-Philippe Aumasson <jpnossa@free.fr>
 * Copyright 2005 Florent Kaisser <jpnossa@free.fr>
 * Copyright 2005 Jean-Baptiste Langlois <florent.kaisser@free.fr>
 *
 * This file is part of SIMS : SIMS Is a Machine Simulator
 *
 * SIMS is free software; you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation; either version 2 of the License, or
 * (at your option) any later version.
 *
 * SIMS is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with SIMS; if not, write to the Free Software
 * Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
 *
 * For more details see the file COPYING.
 */
class GraphState {
    protected static final int Taille = 20;
    protected float rank;
    protected float order;
    protected float CenterX;
    protected float CenterY;
    protected int Numero;
    protected int nbSucc;
    protected int nbPred;
    protected int[] Succ = new int[Taille];
    protected int[] Pred = new int[Taille];
    protected boolean SelfSucc;

    public GraphState(int no) {
        Numero = no;
        SelfSucc = false;
    }

    public int getID() {
        return Numero;
    }

    public void setRank(float x) {
        rank = x;
    }

    public float getRank() {
        return rank;
    }

    public void setOrder(float x) {
        order = x;
    }

    public float getOrder() {
        return order;
    }

    public void addSucc(int i) {
        Succ[nbSucc++] = i;
    }

    public void addPred(int i) {
        Pred[nbPred++] = i;
    }

    public boolean isSucc(int i) {
        for (int j = 0; j < nbSucc; j++)
            if (Succ[j] == i) {
                return true;
            }

        return false;
    }

    public boolean isPred(int i) {
        for (int j = 0; j < nbPred; j++)
            if (Pred[j] == i) {
                return true;
            }

        return false;
    }

    public int getInEdge() {
        //if (isPred(this.getID())) return nbPred-1; else 
        return nbPred;
    }

    public int getOutEdge() {
        //if (isSucc(this.getID())) return nbSucc-1; else 
        return nbSucc;
    }

    public int getSucc(int i) {
        return Succ[i];
    }

    public int getPred(int i) {
        return Pred[i];
    }

    public int getAdjacent(int i) {
        if (i >= nbSucc) {
            return getPred(i - nbSucc);
        } else {
            return getSucc(i);
        }
    }

    public void isSelfSucc(boolean val) {
        SelfSucc = val;
    }

    public boolean getSelfSucc() {
        return SelfSucc;
    }

    public int nbAdjacent() {
        return nbSucc + nbPred;
    }

    public boolean isAdjacent(int j) {
        for (int i = 0; i < nbAdjacent(); i++)
            if (getAdjacent(i) == j) {
                return true;
            }

        return false;
    }
}
