package sims.ihm.graph;


/*
 * Copyright 2005 Jean-Philippe Aumasson <jpnossa@free.fr>
 * Copyright 2005 Florent Kaisser <jpnossa@free.fr>
 * Copyright 2005 Jean-Baptiste Langlois <florent.kaisser@free.fr>
 *
 * This file is part of SIMS : SIMS Is a Machine Simulator
 *
 * SIMS is free software; you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation; either version 2 of the License, or
 * (at your option) any later version.
 *
 * SIMS is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with SIMS; if not, write to the Free Software
 * Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
 *
 * For more details see the file COPYING.
 */
import java.awt.*;
import java.awt.event.*;
import java.awt.geom.*;

import javax.swing.*;


public class GraphState2D extends GraphState {
    private String Value;
    private boolean EndOfGraph;
    private int RadiusH;
    private int RadiusW;
    private int FontSize;
    private Graphics2D graph;
    private boolean presentState;

    public GraphState2D(int x, int y, String name, boolean fin) {
        super(1);
        presentState = false;
        CenterX = x;
        CenterY = y;
        Value = name;
        EndOfGraph = fin;
        RadiusH = 30;
        RadiusW = 30;
        FontSize = 18;

        if ((Value.charAt(0) != 'R') &&
                ((Value.charAt(Value.length() - 1) != '+') ||
                (Value.charAt(Value.length() - 1) != '-'))) {
            RadiusW += ((FontSize / 3) * (Value.length() - 1));
        }
    }

    public GraphState2D(int id) {
        super(id);
        presentState = false;
        Value = "" + id;
        EndOfGraph = false;
        RadiusH = 35;
        RadiusW = 35;
        FontSize = 14;
    }

    public GraphState2D(int id, String s, int taille) {
        super(id);

        int sizeW;
        presentState = false;
        Value = s;
        EndOfGraph = false;
        RadiusH = (int) ((taille * 40) / 600);
        sizeW = s.length() - 3;

        if (sizeW <= 0) {
            sizeW = 40;
        } else {
            sizeW = 40 + (5 * sizeW);
        }

        RadiusW = (int) ((taille * sizeW) / 600);
        FontSize = (int) ((taille * 16) / 600) + 1;

        //System.out.println("Etat : "+s);
    }

    public GraphState2D(int x, int y, int rayon, String name, boolean fin) {
        // If boolean fin is true, the state is double-circled.
        super(1);
        presentState = false;
        CenterX = x;
        CenterY = y;
        Value = name;
        EndOfGraph = fin;
        RadiusH = rayon;
        RadiusW = rayon;
        FontSize = 18;

        if ((Value.charAt(0) != 'R') &&
                ((Value.charAt(Value.length() - 1) != '+') ||
                (Value.charAt(Value.length() - 1) != '-'))) {
            RadiusW += ((FontSize / 3) * (Value.length() - 1));
        }
    }

    public void GraphPlace(Graphics2D g2) {
        graph = g2;
    }

    public float getX() {
        return CenterX;
    }

    public float getY() {
        return CenterY;
    }

    public boolean isFinalState() {
        return EndOfGraph;
    }

    public void FinalState(boolean fin) {
        EndOfGraph = fin;
    }

    public int getRadius() {
        return RadiusH;
    }

    public int getRadiusW() {
        return RadiusW;
    }

    public void setValue(String name) {
        Value = name;
    }

    public String getValue() {
        return Value;
    }

    public void setX(float x) {
        CenterX = x;
    }

    public void setY(float y) {
        CenterY = y;
    }

    public boolean isCurrent() {
        return presentState;
    }

    public void setCurrent(boolean active) {
        presentState = active;
    }

    public void setRadius(int rayon) {
        RadiusH = rayon;
        RadiusW = rayon;
        FontSize = RadiusH / 2;

        if ((Value.charAt(0) != 'R') &&
                ((Value.charAt(Value.length() - 1) != '+') ||
                (Value.charAt(Value.length() - 1) != '-'))) {
            RadiusW += ((FontSize / 3) * (Value.length() - 1));
        }
    }

    public void setFontSize(int size) {
        FontSize = size;
    }

    public int getFontSize() {
        return FontSize;
    }

    public void draw() {
        float TextX;
        float TextY;

        //  System.out.println("Etat : "+ Value+ ", present ? "+presentState);
        if (!presentState) {
            //System.out.println(
            //        "WHITE value:" + Value + " present " + presentState);
            graph.setPaint(Color.white);
            graph.fill(new Ellipse2D.Double(CenterX - (RadiusW / 2),
                    CenterY - (RadiusH / 2), RadiusW, RadiusH));
        } else {
            graph.setPaint(Color.RED);

            // System.out.println(
            //         "RED value:" + Value + " present " + presentState);
            graph.fill(new Ellipse2D.Double(CenterX - (RadiusW / 2),
                    CenterY - (RadiusH / 2), RadiusW, RadiusH));
        }

        graph.setPaint(Color.black);
        graph.draw(new Ellipse2D.Double(CenterX - (RadiusW / 2),
                CenterY - (RadiusH / 2), RadiusW, RadiusH));

        if (EndOfGraph) {
            graph.draw(new Ellipse2D.Double(CenterX - (RadiusW / 2) + 3,
                    CenterY - (RadiusH / 2) + 3, RadiusW - 6, RadiusH - 6));
        }

        graph.setFont(new Font("SansSerif", Font.PLAIN, FontSize));
        TextY = CenterY + (FontSize / 3);

        if ((Value.charAt(0) != 'R') &&
                ((Value.charAt(Value.length() - 1) != '+') ||
                (Value.charAt(Value.length() - 1) != '-'))) {
            TextX = CenterX - ((Value.length() * FontSize) / 4);
            graph.drawString(Value, TextX, TextY);
        } else {
            TextX = CenterX - (FontSize / 3);
            graph.drawString(Value.charAt(0) + " ", TextX - (FontSize / 4),
                TextY);
            graph.setFont(new Font("SansSerif", Font.PLAIN, FontSize));
            graph.drawString(Value.charAt(Value.length() - 1) + " ",
                TextX + (FontSize / 2), TextY - (FontSize / 2));
            graph.drawString(Value.substring(1, Value.length() - 1),
                TextX + (FontSize / 2), TextY + (FontSize / 4));
        }
    }
}
