package sims.ihm.graph;


/*
 * Copyright 2005 Jean-Philippe Aumasson <jpnossa@free.fr>
 * Copyright 2005 Florent Kaisser <jpnossa@free.fr>
 * Copyright 2005 Jean-Baptiste Langlois <florent.kaisser@free.fr>
 *
 * This file is part of SIMS : SIMS Is a Machine Simulator
 *
 * SIMS is free software; you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation; either version 2 of the License, or
 * (at your option) any later version.
 *
 * SIMS is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with SIMS; if not, write to the Free Software
 * Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
 *
 * For more details see the file COPYING.
 */

/**
*  Class to calculate the rank to define the Y-coordinates of the vertices.
*  @author Jean-Baptiste Langlois
*  @version 1.12
*  Rank.java
*/
class Rank {
    private Graph g;

    /**
    *  Rank Constructor to calculate the rank
    *  @param GraphDestination
    *  Rank()
    */
    public Rank(Graph ici) {
        g = ici;
    }

    /**
    * Return the smallest rank in that graph
    * @return the minimum rank
    * minimumRank()
    */
    public float minimumRank() {
        float i = g.getState(0).getRank();

        for (int j = 0; j < g.nbStates; j++)
            if (g.getState(j).getRank() < i) {
                i = g.getState(j).getRank();
            }

        return i;
    }

    /**
    * Return the greatest rank in that graph
    * @return the maximum rank
    * maximumRank()
    */
    public float maximumRank() {
        float i = g.getState(0).getRank();

        for (int j = 0; j < g.nbStates; j++)
            if (g.getState(j).getRank() > i) {
                i = g.getState(j).getRank();
            }

        if (i < 1) {
            i = 1;
        }

        return i;
    }

    /**
    * Calculate the initial rank of the graph
    * @param initialVertex
    * initRanks()
    */
    public void initRanks(int racine) {
        boolean[] tab = new boolean[g.nbStates];

        for (int i = 0; i < g.nbStates; i++)
            tab[i] = false;

        boolean tous = false;
        int i = racine;
        FileA f = new FileA();
        f.enfiler(i);

        //System.out.println(racine);
        g.getState(racine).setRank(g.getState(racine).getOutEdge());

        while (!tous) {
            //System.out.println("-->"+i);
            i = f.tete();
            tab[i] = true;
            f.defiler();

            for (int j = 0; j < g.getState(i).getOutEdge(); j++) {
                g.getState(g.getState(i).getSucc(j)).setRank((g.getState(i)
                                                               .getRank() * (j +
                    1)) / g.getState(i).getOutEdge());
                f.enfiler(g.getState(i).getSucc(j));
            }

            tous = true;

            for (int j = 0; j < g.nbStates; j++)
                if (!tab[j]) {
                    //System.out.println("manque :"+j);
                    tous = false;
                }
        }
    }

    public int[] tabRank(int j) {
        int tot = 0;

        for (int i = 0; i < g.nbStates; i++)
            if (g.getState(i).getRank() == j) {
                tot++;
            }

        int[] tab = new int[tot];
        tot = 0;

        for (int i = 0; i < g.nbStates; i++)
            if (g.getState(i).getRank() == j) {
                tab[tot++] = i;
            }

        return tab;
    }

    /**
    * Multiply all the ranks to get only integer numbers
    * normalize()
    */
    public void normalize() {
        for (int k = 0; k < g.nbStates; k++)
            if ((g.getState(k).getRank() > 0) && (g.getState(k).getRank() < 1)) {
                int j = (int) (1 / g.getState(k).getRank());

                for (int i = 0; i < g.nbStates; i++)
                    g.getState(i).setRank(g.getState(i).getRank() * j);
            }
    }

    /**
    * Substract all the ranks until one reach zero to get the smallest ranks
    * normalizeByZero()
    */
    public void normalizeByZero() {
        float i = minimumRank();

        for (int j = 0; j < g.nbStates; j++)
            g.getState(j).setRank(g.getState(j).getRank() - i);
    }

    public int depthSearch(int ici, int valeur, boolean[] visite) {
        int sum = 0;

        if (g.getState(ici).getOutEdge() != 0) {
            for (int i = 0; i < g.getState(ici).getOutEdge(); i++)
                sum = depthSearch(g.getState(ici).getSucc(i), valeur + i, visite);
        } else {
            sum = valeur;
        }

        if (!visite[ici]) {
            g.getState(ici).setRank(g.getState(ici).getRank() + sum);
        }

        visite[ici] = true;

        return sum + 1;
    }
}
