package sims.ihm.graph;


/*
 * Copyright 2005 Jean-Philippe Aumasson <jpnossa@free.fr>
 * Copyright 2005 Florent Kaisser <jpnossa@free.fr>
 * Copyright 2005 Jean-Baptiste Langlois <florent.kaisser@free.fr>
 *
 * This file is part of SIMS : SIMS Is a Machine Simulator
 *
 * SIMS is free software; you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation; either version 2 of the License, or
 * (at your option) any later version.
 *
 * SIMS is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with SIMS; if not, write to the Free Software
 * Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
 *
 * For more details see the file COPYING.
 */
import java.awt.*;
import java.awt.event.*;
import java.awt.geom.*;

import javax.swing.*;


/**
*  Class to draw a transition between two vertices.
*  @author Jean-Baptiste Langlois
*  @version 1.0alpha2
*  Transition2D.java
*/
class Transition2D {
    private Graphics2D graph;
    private float X;
    private GraphState2D state1;
    private GraphState2D state2;
    private float Y;
    private String label;
    private int fontSize;
    private Arrow2D arw;
    private Graph gr;

    /**
    *  Transition2D Constructor to display a transition between two vertices.
    *  @param GraphDestination
    *  @param Arrow
    *  @param StringLabel
    *  @param Size
    *  Transition2D()
    */
    public Transition2D(Graph gr, Arrow2D arw, String label, int taille) {
        this.graph = gr.getGraphics();
        this.gr = gr;
        this.label = label;
        this.state1 = arw.getState1();
        this.state2 = arw.getState2();
        this.arw = arw;
        fontSize = (int) ((taille * 12) / 600) + 1;
    }

    /**
    *  First accessor to get the equation of the line y=mx+p
    *  @return the coefficient of the line
    *  m()
    */
    public float m(GraphState2D s1, GraphState2D s2) {
        return ((s2.getY() - s1.getY()) / (s2.getX() - s1.getX()));
    }

    /**
    *  Second accessor to get the equation of the line y=mx+p
    *  @return the constant of the equation
    *  p()
    */
    public float p(float m, GraphState2D s) {
        return s.getY() - (m * s.getX());
    }

    /**
    *  Calculate the coordinates where to display the label
    *  calculate()
    */
    public void calculate() {
        float dm;
        float dp;

        if (label != null) {
            if ((state1.getX() < state2.getX()) &&
                    (state2.getY() < state1.getY())) {
                dm = m(state1, state2);
                dp = p(dm, state1);
                this.Y = ((state1.getY() + state2.getY()) / 2);
                this.X = fontSize + ((this.Y - dp) / dm);

                /*this.X = -(label.length() * 4) +
                    ((state1.getX() + state2.getX()) / 2);
                this.Y = -10 + ((state1.getY() + state2.getY()) / 2);*/
            }

            if ((state1.getX() < state2.getX()) &&
                    (state2.getY() > state1.getY())) {
                dm = m(state1, state2);
                dp = p(dm, state1);
                this.Y = ((state1.getY() + state2.getY()) / 2);

                if (gr.isSucc(state2, state1)) {
                    this.X = (-2 * fontSize) -
                        ((label.length() * fontSize) / 3) +
                        ((this.Y - dp) / dm);
                } else {
                    this.X = fontSize + ((this.Y - dp) / dm);
                }

                /*this.X = -(label.length() * 4) +
                    ((state1.getX() + state2.getX()) / 2);
                this.Y = 10 + ((state1.getY() + state2.getY()) / 2);*/
            }

            if ((state1.getX() > state2.getX()) &&
                    (state2.getY() < state1.getY())) {
                dm = m(state1, state2);
                dp = p(dm, state1);
                this.Y = ((state1.getY() + state2.getY()) / 2);
                this.X = fontSize + ((this.Y - dp) / dm);

                /*this.X = (label.length() * 4) +
                    ((state1.getX() + state2.getX()) / 2);
                this.Y = -10 + ((state1.getY() + state2.getY()) / 2);*/
            }

            if ((state1.getX() > state2.getX()) &&
                    (state2.getY() > state1.getY())) {
                /*this.X = (label.length() * 4) +
                    ((state1.getX() + state2.getX()) / 2);
                this.Y = 10 + ((state1.getY() + state2.getY()) / 2);*/
                dm = m(state1, state2);
                dp = p(dm, state1);
                this.Y = ((state1.getY() + state2.getY()) / 2);

                if (gr.isSucc(state2, state1)) {
                    this.X = (-2 * fontSize) -
                        ((label.length() * fontSize) / 3) +
                        ((this.Y - dp) / dm);
                } else {
                    this.X = fontSize + ((this.Y - dp) / dm);
                }
            }

            if ((state1.getX() < state2.getX()) &&
                    (state2.getY() == state1.getY())) {
                this.X = (-label.length() * (fontSize / 6)) +
                    ((state1.getX() + state2.getX()) / 2);
                this.Y = state1.getY() - (state1.getRadius() / 4);
            }

            if ((state1.getX() == state2.getX()) &&
                    (state2.getY() < state1.getY())) {
                this.X = fontSize + state1.getX();
                this.Y = (state1.getY() + state2.getY()) / 2;
            }

            if ((state1.getX() == state2.getX()) &&
                    (state2.getY() > state1.getY())) {
                if (gr.isSucc(state2, state1)) {
                    this.X = (-label.length() * (fontSize / 3)) +
                        state1.getX();
                } else {
                    this.X = fontSize + state1.getX();
                }

                this.Y = (state1.getY() + state2.getY()) / 2;
            }

            if ((state1.getX() > state2.getX()) &&
                    (state2.getY() == state1.getY())) {
                this.X = (-label.length() * (fontSize / 6)) +
                    ((state1.getX() + state2.getX()) / 2);
                this.Y = state1.getY() + (state1.getRadius() / 4);
            }

            if ((state1.getX() == state2.getX()) &&
                    (state2.getY() == state1.getY())) {
                /*this.X = state1.getX() + state1.getRadiusW();
                this.Y = state1.getY() + state2.getRadius();*/
                if (arw.getDirection() == 0) {
                    this.X = state1.getX() -
                        ((this.label.length() * fontSize) / 6);
                    this.Y = state1.getY() - state1.getRadius();
                }

                if (arw.getDirection() == 2) {
                    this.X = state1.getX() -
                        ((this.label.length() * fontSize) / 3);
                    this.Y = state1.getY() - state1.getRadius();
                }

                if (arw.getDirection() == 1) {
                    this.X = state1.getX() -
                        ((this.label.length() * fontSize) / 6);
                    this.Y = state1.getY() + state1.getRadius();
                }

                if (arw.getDirection() == 3) {
                    this.X = state1.getX() -
                        ((this.label.length() * fontSize) / 3);
                    this.Y = state1.getY() + state1.getRadius();
                }
            }
        }

        //this.X=(state1.getX()+state2.getX())/2;
        //      this.Y=(state1.getY()+state2.getY())/2;
    }

    public String getLabel() {
        return label;
    }

    public float getX() {
        return X;
    }

    public float getY() {
        return Y;
    }

    /**
    *  Display the label where it must be
    *  draw()
    */
    public void draw() {
        if (label != null) {
            graph.setFont(new Font("SansSerif", Font.PLAIN, fontSize));
            graph.drawString(label, X, Y);
        }
    }
}
