//$Id: FenetreRegistres.java 1.2 2004/11/28 21:41:13 Florent Exp Florent $
package sims.ihm.mar;


/*
 * Copyright 2005 Jean-Philippe Aumasson <jpnossa@free.fr>
 * Copyright 2005 Florent Kaisser <jpnossa@free.fr>
 * Copyright 2005 Jean-Baptiste Langlois <florent.kaisser@free.fr>
 *
 * This file is part of SIMS : SIMS Is a Machine Simulator
 *
 * SIMS is free software; you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation; either version 2 of the License, or
 * (at your option) any later version.
 *
 * SIMS is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with SIMS; if not, write to the Free Software
 * Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
 *
 * For more details see the file COPYING.
 */
import sims.mar.*;

import java.awt.*;

import javax.swing.*;
import javax.swing.table.*;


/**
 *
 * Fenetre permetant de visualiser la valeur des registres
 *
 * @author florent.kaisser@free.fr
 * @version $Id: FenetreRegistres.java 1.2 2004/11/28 21:41:13 Florent Exp Florent $
 *
 * $Log: FenetreRegistres.java $
 * Revision 1.2  2004/11/28 21:41:13  Florent
 * Ajout d'un titre  la fenetre
 *
 * Revision 1.1  2004/11/27 15:28:59  Florent
 * Initial revision
 *
 */
public class FenetreRegistres extends JToolBar {
    MachineARegistre machine; // Collection de Registre
    JTable table = null;

    /**
     *
     * @param machine Machine  registre, dont on veux afficher ses valeurs des registres
     */
    public FenetreRegistres(MachineARegistre machine) {
        super("Registers");
        this.machine = machine;

        table = new JTable(new ModeleRegistre());

        //table.setCellEditor(new RegistreEditor());
        //table.setDefaultRenderer(String.class, new AfficheurCelluleCouleur());
        table.setPreferredScrollableViewportSize(new Dimension(300, 70));

        JPanel pan = new JPanel(new BorderLayout());

        pan.add(table.getTableHeader(), BorderLayout.PAGE_START);
        pan.add(new JScrollPane(table), BorderLayout.CENTER);

        add(pan);

        setVisible(true);
    }

    /**
     * Actualise la valeur des registre affichs
     *
     */
    public void actualiser() {
        ((ModeleRegistre) table.getModel()).fireTableDataChanged();
    }

    /**
     * Modele pour le JTable
     */
    public class ModeleRegistre extends AbstractTableModel {
        public boolean isCellEditable(int row, int col) {
            //seul les valeurs de la colone 1 peuvent etre modifi
            return col == 1;
        }

        public int getColumnCount() {
            //On  toujours 5 colones
            return 5;
        }

        public void setValueAt(Object value, int row, int col) {
            if (col == 1) {
                //Dfinie la valeur du registre associe  la cellule row , col
                machine.getProgramme().getRegistre(row + 1).setValeur(Integer.parseInt(
                        (String) value));
                fireTableCellUpdated(row, col);
            }
        }

        //Calcule la sommet du nombre de fois que les registres ont t decrment
        private int totalDecRegistre() {
            int total = 0;

            for (int i = 1; i <= machine.getProgramme().getNbrRegistre();
                    i++)
                total += machine.getProgramme().getRegistre(i).getNbrDec();

            return total;
        }

        //Calcule la sommet du nombre de fois que les registres ont t incrment
        private int totalIncRegistre() {
            int total = 0;

            for (int i = 1; i <= machine.getProgramme().getNbrRegistre();
                    i++)
                total += machine.getProgramme().getRegistre(i).getNbrInc();

            return total;
        }

        public Object getValueAt(int parm1, int parm2) {
            //si c'est pas la ligne des totaux
            if (parm1 < machine.getProgramme().getNbrRegistre()) {
                Registre r = machine.getProgramme().getRegistre(parm1 + 1);

                //selon la collone
                switch (parm2) {
                case 0:

                    //Intitul des registres
                    return r.intitule();

                case 1:

                    //Valeur du registre
                    return r.toString();

                case 2:

                    //Valeur maximum atteinte par un registre
                    return String.valueOf(r.getMaxValeur());

                case 3:

                    //nombre de dcrementation sur un registre
                    return String.valueOf(r.getNbrDec());

                case 4:

                    //nombre d'incrementation sur un registre
                    return String.valueOf(r.getNbrInc());

                default:
                    return r;
                }
            } else {
                switch (parm2) {
                case 2:
                    return "Total :";

                case 3:
                    return String.valueOf(totalDecRegistre());

                case 4:
                    return String.valueOf(totalIncRegistre());

                default:
                    return "";
                }
            }
        }

        public int getRowCount() {
            if (machine.getProgramme() == null) {
                return 0;
            } else {
                //le nombre de ligne, est le nombre de registre + la ligne des totaux
                return machine.getProgramme().getNbrRegistre() + 1;
            }
        }

        public String getColumnName(int col) {
            switch (col) {
            case 0:
                return "Register";

            case 1:
                return "Value";

            case 2:
                return "Max";

            case 3:
                return "DecNb";

            case 4:
                return "IncNd";
            }

            return "NA";
        }
    }

    public class AfficheurCelluleCouleur extends DefaultTableCellRenderer {
        public Component getTableCellRendererComponent(JTable table,
            Object value, boolean isSelected, boolean hasFocus, int row, int col) {
            super.getTableCellRendererComponent(table, value, isSelected,
                hasFocus, row, col);

            setBackground(Color.blue);

            return this;
        }
    }
}
