//$Id: Main.java 1.2 2004/11/28 21:41:13 Florent Exp Florent $
package sims.ihm.mar;

import sims.*;

import sims.ihm.*;

import sims.ihm.graph.*;

import sims.mar.*;

import java.awt.*;
import java.awt.event.*;

import java.io.*;

import java.util.*;

import javax.swing.*;
import javax.swing.event.*;
import javax.swing.text.*;
import javax.swing.undo.*;


/**
 *
 * Fenetre principale pour l'diteur d'une machine  registre
 *
 * @author florent.kaisser@free.fr
 * @version $Id: Main.java 1.2 2004/11/28 21:41:13 Florent Exp Florent $
 *
 * $Log: Main.java $
 * Revision 1.2  2004/11/28 21:41:13  Florent
 * Modification pour gerer les nouvelles exception
 * Ajout de surbrillance d'une erreur de compilation
 *
 * Revision 1.1  2004/11/27 15:28:59  Florent
 * Initial revision
 *
 */
public class Main extends JFrame implements ActionMAR, WindowListener,
    ActionListener, DocumentListener {
    /*
     * Constante identifiant chaque commande
     */
    static final private String NOUVEAU = "nouveau";
    static final private String OUVRIR = "ouvrir";
    static final private String ENREGISTRER = "enregistrer";
    static final private String COMPILER = "compiler";
    static final private String ENREGISTRER_SOUS = "enregistrer_sous";
    static final private String QUITTER = "quitter";
    static final private String FENETRE_REGISTRE = "fenetre_registre";
    static final private String FENETRE_MESSAGE = "fenetre_message";
    static final private String FENETRE_GRAPHE = "fenetre_graphe";
    static final private String FENETRE_COMMANDE = "fenetre_commande";
    private MachineARegistre machine; //Machine  registre associ  l'editeur
    private FenetreRegistres fRegistre; //ToolBar pour affiche les registre
    private Commandes c; //ToolBar des commande du simulateur
    private Editeur e; //Editeur
    private FenetreMessage fm; //ToolBar pour afficher les messages
    private JToolBar tb; //ToolBar des boutons de l'editeur
    private File file; //Fichier ouvert par l'editeur
    private JPanel pNord; //Panel Nord
    private JPanel pSud; //Panel Sud
    private JGraph2D jGraphe; //Fenetre d'affichage des graphe
    private int oldLigne = -1;

    //undo helpers
    protected UndoAction undoAction;
    protected RedoAction redoAction;
    protected UndoManager undo = new UndoManager();
    HashMap actions;

    public Main(String pathFile) {
        super("Editeur de machine  registre : " + pathFile);

        //obtient la taille de la l'ecran
        Toolkit k = Toolkit.getDefaultToolkit();
        Dimension tailleEcran = k.getScreenSize();
        int largeurEcran = tailleEcran.width;
        int hauteurEcran = tailleEcran.height;

        //Initialise la machine
        file = new File(pathFile);

        //Initialise l'editeur
        e = new Editeur();

        fm = new FenetreMessage();

        machine = new MachineARegistre();

        //Si le fichier existe, le fait lire par l'editeur ,et la machine
        if (file.exists()) {
            try {
                e.read(file);
                machine.charger(file);
            } catch (ExceptionCompilation ec) {
                afficherErreurDeSyntaxe(ec);
            } catch (IOException ioe) {
                System.out.println(ioe.getMessage());
            }

            //sinon, cre un nouveau fichier
        } else {
            try {
                file.createNewFile();
            } catch (IOException ioe) {
                System.out.println(ioe.getMessage());
            }
        }

        e.setMargin(new Insets(0, 10, 0, 0));
        e.getDocument().addDocumentListener(this);
        e.getDocument().addUndoableEditListener(new MyUndoableEditListener());

        //Cre la barre d'outils
        tb = new JToolBar("Commande d'dition");
        ajouterBoutonBarreOutils();

        c = new Commandes(machine);
        c.enableButtons(machine.getProgramme() != null);

        fRegistre = new FenetreRegistres(machine);

        pNord = new JPanel(new FlowLayout(FlowLayout.LEFT));
        pSud = new JPanel(new GridLayout(2, 1));

        //ajoute les composant
        getContentPane().setLayout(new BorderLayout());
        getContentPane().add(new JScrollPane(e), BorderLayout.CENTER);

        //ajoute les panels
        pNord.add(tb);
        pNord.add(c);
        pSud.add(fRegistre);
        pSud.add(fm);

        getContentPane().add(pNord, BorderLayout.NORTH);
        getContentPane().add(pSud, BorderLayout.SOUTH);

        //initialise la'ffichage des graphe
        jGraphe = new JGraph2D(machine.getProgramme());
        jGraphe.setBounds(largeurEcran / 2, 0, largeurEcran / 2,
            hauteurEcran - GraphicValues.TAILLE_BARRE_TACHE);

        jGraphe.setVisible(true);

        //definie la classe que la machine doit appeler, pour notifier des venements
        machine.setAction(this);

        //Actualise les info de debugage
        actualiserInfoDebug();

        //Menu
        creerActionTable(e);

        JMenuBar mb = new JMenuBar();
        mb.add(creerMenuFichier());
        mb.add(creerMenuEdition());
        setJMenuBar(mb);

        //finalise la fenetre
        addWindowListener(this);
        pack();

        //setLocation(300,300);	
        setBounds(0, 0, largeurEcran / 2,
            hauteurEcran - GraphicValues.TAILLE_BARRE_TACHE);
        setVisible(true);
    }

    //affiche les info sur l'erreur de syntaxe
    private void afficherErreurDeSyntaxe(ExceptionCompilation ec) {
        fm.addMessage("Erreur de syntaxe : " + ec.getMessage() +
            "  la ligne " + ec.getLigne());
        e.setLigneSurbrillance(ec.getLigne(), Color.RED);
    }

    //Actualise les information sur l'etat de la machi,e
    private void actualiserInfoDebug() {
        if (machine.getProgramme() != null) {
            fRegistre.actualiser();

            int ligneProchianeInst = machine.getProgramme()
                                            .getInstructionCourante().getLigne();

            jGraphe.setEtatCourant(machine.getProgramme()
                                          .getInstructionCourante());

            if (ligneProchianeInst != -1) {
                e.setLigneSurbrillance(ligneProchianeInst, Color.YELLOW);
            } else {
                e.enleverSurbrillance();
            }
        }
    }

    public void instructionExecute(Instruction i) {
        if ((machine.getVitesse() != 0) || machine.estArrete()) {
            actualiserInfoDebug();
            System.out.println(i.getLigne() + " " + i.getCommande());
        }
    }

    public void programmeBreakPoint() {
        fm.addMessage("Breakpoint");
        actualiserInfoDebug();
        c.resetButtonGo();
    }

    public void programmeFin() {
        c.resetButtonGo();
        System.out.println("fin");
        actualiserInfoDebug();
        fm.addMessage("program end");
    }

    public void programmeDebut() {
        actualiserInfoDebug();
        fm.addMessage("program start");
        c.resetButtonGo();
        jGraphe.setMachine(machine.getProgramme());
    }

    public void vitesseChange(int vitesse) {
        fm.addMessage("Speed : " + vitesse);
    }

    /*-------------------------------------------------------------
     *
     * partie dedie  la gestion du menu et de ses commandes associes
     *
     */
    private void creerActionTable(JTextComponent textComponent) {
        actions = new HashMap();

        Action[] actionsArray = textComponent.getActions();

        for (int i = 0; i < actionsArray.length; i++) {
            Action a = actionsArray[i];
            actions.put(a.getValue(Action.NAME), a);
        }
    }

    private Action getActionByName(String name) {
        return (Action) (actions.get(name));
    }

    private void ajouterMenuItemEdition(JMenu menu, String label, String cmd,
        KeyStroke k) {
        JMenuItem i = menu.add(getActionByName(cmd));
        i.setText(label);
        i.setAccelerator(k);
    }

    //Create the edit menu.
    protected JMenu creerMenuEdition() {
        JMenu menu = new JMenu("Edition");

        //Undo and redo are actions of our own creation.
        undoAction = new UndoAction();
        menu.add(undoAction).setAccelerator(KeyStroke.getKeyStroke(
                KeyEvent.VK_Z, InputEvent.CTRL_MASK));

        redoAction = new RedoAction();
        menu.add(redoAction).setAccelerator(KeyStroke.getKeyStroke(
                KeyEvent.VK_Y, InputEvent.CTRL_MASK));

        menu.addSeparator();

        //These actions come from the default editor kit.
        //Get the ones we want and stick them in the menu.
        ajouterMenuItemEdition(menu, "Cut", DefaultEditorKit.cutAction,
            KeyStroke.getKeyStroke(KeyEvent.VK_C, InputEvent.CTRL_MASK));
        ajouterMenuItemEdition(menu, "Copy", DefaultEditorKit.copyAction,
            KeyStroke.getKeyStroke(KeyEvent.VK_X, InputEvent.CTRL_MASK));
        ajouterMenuItemEdition(menu, "Paste", DefaultEditorKit.pasteAction,
            KeyStroke.getKeyStroke(KeyEvent.VK_V, InputEvent.CTRL_MASK));

        menu.addSeparator();

        ajouterMenuItemEdition(menu, "Select all",
            DefaultEditorKit.selectAllAction,
            KeyStroke.getKeyStroke(KeyEvent.VK_A, InputEvent.CTRL_MASK));

        return menu;
    }

    private void ajouterMenuItemFichier(JMenu menu, String label, String cmd,
        KeyStroke k) {
        JMenuItem i = menu.add(label);
        i.setActionCommand(cmd);
        i.addActionListener(this);
        i.setAccelerator(k);
    }

    //Create the edit menu.
    protected JMenu creerMenuFichier() {
        JMenu menu = new JMenu("File");

        //These actions come from the default editor kit.
        //Get the ones we want and stick them in the menu.
        ajouterMenuItemFichier(menu, "New", NOUVEAU,
            KeyStroke.getKeyStroke(KeyEvent.VK_N, InputEvent.CTRL_MASK));
        ajouterMenuItemFichier(menu, "Open", OUVRIR,
            KeyStroke.getKeyStroke(KeyEvent.VK_O, InputEvent.CTRL_MASK));

        ajouterMenuItemFichier(menu, "Save", ENREGISTRER,
            KeyStroke.getKeyStroke(KeyEvent.VK_S, InputEvent.CTRL_MASK));
        ajouterMenuItemFichier(menu, "Save as...", ENREGISTRER_SOUS,
            KeyStroke.getKeyStroke(KeyEvent.VK_S, InputEvent.ALT_MASK));
        menu.addSeparator();
        ajouterMenuItemFichier(menu, "Compile", COMPILER,
            KeyStroke.getKeyStroke(KeyEvent.VK_F5, 0));
        menu.addSeparator();
        ajouterMenuItemFichier(menu, "Quit", QUITTER,
            KeyStroke.getKeyStroke(KeyEvent.VK_Q, InputEvent.CTRL_MASK));

        return menu;
    }

    /*Fin de la partie de la gestion du menu
     * -----------------------------------------------------------------------------
     */
    /*
     * Cre un bouton
     */
    protected JButton creerBouton(String imageName, String actionCommand,
        String toolTipText, String altText) {
        //Charge l'image
        String imgLocation = "sims/ihm/visuals/" + imageName + ".png";
        Icon icon = new ImageIcon(imgLocation);

        //cre et initialise le bouton
        JButton button = new JButton();
        button.setActionCommand(actionCommand);
        button.setToolTipText(toolTipText);
        button.addActionListener(this);

        if (icon != null) { //image trouv
            button.setIcon(icon);
        } else { //image non trouv
            button.setText(altText);
            System.err.println("Resource not found: " + imgLocation);
        }

        return button;
    }

    /*
     * Cre un bouton  deux etats
     */
    protected JToggleButton creerToggleBouton(String imageName,
        String actionCommand, String toolTipText, String altText, boolean etat) {
        //Charge l'image
        String imgLocation = "sims/ihm/visuals/" + imageName + ".png";
        Icon icon = new ImageIcon(imgLocation);

        //cre et initialise le bouton
        JToggleButton button = new JToggleButton();
        button.setActionCommand(actionCommand);
        button.setToolTipText(toolTipText);
        button.addActionListener(this);

        if (icon != null) { //image trouv
            button.setIcon(icon);
        } else { //image non trouv
            button.setText(altText);
            System.err.println("Resource not found: " + imgLocation);
        }

        button.setSelected(etat);

        return button;
    }

    //Ajoute les bouton de la barre d'outil
    private void ajouterBoutonBarreOutils() {
        tb.add(creerBouton("save32", ENREGISTRER,
                "Enregistre le fichier en cours d'dition", "E"));
        tb.add(creerBouton("comp", COMPILER, "Compile la machine", "C"));
        tb.addSeparator();
        tb.add(creerToggleBouton("fenreg", FENETRE_REGISTRE,
                "Show/hide registers window", "R", true));
        tb.add(creerToggleBouton("fenmsg", FENETRE_MESSAGE,
                "Show/hide message window", "R", true));
        tb.add(creerToggleBouton("fencmd", FENETRE_COMMANDE,
                "Show/hide commands window", "R", true));
        tb.add(creerToggleBouton("fengraphe", FENETRE_GRAPHE,
                "Show/hide graph window", "R", true));
    }

    public void windowActivated(WindowEvent e) {
    }

    public void windowClosed(WindowEvent e) {
    }

    public void windowDeactivated(WindowEvent e) {
    }

    public void windowDeiconified(WindowEvent e) {
    }

    public void windowIconified(WindowEvent e) {
    }

    public void windowOpened(WindowEvent e) {
    }

    public void windowClosing(WindowEvent e) {
        machine.tuer();
        setVisible(false);
        jGraphe.setVisible(false);
        dispose();

        try {
            finalize();
        } catch (Throwable t) {
        }

        //System.exit(0);
    }

    public void insertUpdate(DocumentEvent e) {
        //quand on modifi le programme : on ne peux plus l'executer, tant qu'on la pas recompil
        c.enableButtons(false);
    }

    public void removeUpdate(DocumentEvent e) {
        //quand on modifi le programme : on ne peux plus l'executer, tant qu'on la pas recompil
        c.enableButtons(false);
    }

    public void changedUpdate(DocumentEvent e) {
    }

    //enregistre le contenue de l'editeur
    private void enregistrer() {
        try {
            e.write(file);
        } catch (IOException ioe) {
        }
    }

    /**
     * gestion de l'action lies lies aux commandes
     */
    public void actionPerformed(ActionEvent ae) {
        String cmd = ae.getActionCommand();

        if (ENREGISTRER_SOUS.equals(cmd)) {
            JFileChooser fileSelect = new JFileChooser();
            fileSelect.setCurrentDirectory(file.getParentFile());
            fileSelect.setMultiSelectionEnabled(false);
            fileSelect.addChoosableFileFilter(new RegFilter());
            fileSelect.setAcceptAllFileFilterUsed(false);

            int result = fileSelect.showSaveDialog(this);

            if (fileSelect.getSelectedFile() != null) {
                //definie le nouveau fichier courant
                file = fileSelect.getSelectedFile();
                enregistrer();
                setTitle("Register machines editor : " + file.getPath());
            }
        }

        if (QUITTER.equals(cmd)) {
            windowClosing(null);
        }

        if (ENREGISTRER.equals(cmd)) {
            enregistrer();
        }

        if (NOUVEAU.equals(cmd)) {
            JFileChooser fileSelect = new JFileChooser();
            fileSelect.setCurrentDirectory(file.getParentFile());
            fileSelect.setMultiSelectionEnabled(false);
            fileSelect.addChoosableFileFilter(new RegFilter());
            fileSelect.setAcceptAllFileFilterUsed(false);

            int result = fileSelect.showSaveDialog(this);

            if (fileSelect.getSelectedFile() != null) {
                //definie le nouveau fichier courant
                file = fileSelect.getSelectedFile();

                try {
                    file.createNewFile();
                    e.read(file);
                } catch (IOException ioe) {
                    System.out.println(ioe.getMessage());
                }

                machine.decharger();

                c.enableButtons(false);

                actualiserInfoDebug();
                setTitle("Register machines editor : " + file.getPath());
            }
        }

        if (OUVRIR.equals(cmd)) {
            JFileChooser fileSelect = new JFileChooser();
            fileSelect.setCurrentDirectory(file.getParentFile());
            fileSelect.setMultiSelectionEnabled(false);
            fileSelect.addChoosableFileFilter(new RegFilter());
            fileSelect.setAcceptAllFileFilterUsed(false);

            int result = fileSelect.showOpenDialog(this);

            if (fileSelect.getSelectedFile() != null) {
                //definie le nouveau fichier courant
                file = fileSelect.getSelectedFile();

                //Si le fichier existe, le fait lire par l'editeur ,et la machine
                try {
                    e.read(file);
                    machine.charger(file);
                } catch (ExceptionCompilation ec) {
                    afficherErreurDeSyntaxe(ec);
                } catch (IOException ioe) {
                    System.out.println(ioe.getMessage());
                }

                c.enableButtons(machine.getProgramme() != null);
                c.actualiserVitesse();

                actualiserInfoDebug();
                setTitle("Register machines editor : " + file.getPath());
            }
        }

        if (COMPILER.equals(cmd)) {
            //enregistre toujours avant de compiler
            enregistrer();

            try {
                //compile
                machine.charger(file);
            } catch (ExceptionCompilation ec) {
                afficherErreurDeSyntaxe(ec);
            } catch (IOException ioe) {
                System.out.println(ioe.getMessage());
            }

            //Si la compilation c'est bien droul, on rend l'execution possible
            c.enableButtons(machine.getProgramme() != null);
            c.actualiserVitesse();
            e.updateLigne();

            actualiserInfoDebug();
        }

        /*
         * Affiche/cache une fenetre
         */
        if (FENETRE_REGISTRE.equals(cmd)) {
            fRegistre.setVisible(((JToggleButton) ae.getSource()).isSelected());
        }

        if (FENETRE_COMMANDE.equals(cmd)) {
            c.setVisible(((JToggleButton) ae.getSource()).isSelected());
        }

        if (FENETRE_MESSAGE.equals(cmd)) {
            fm.setVisible(((JToggleButton) ae.getSource()).isSelected());
        }

        if (FENETRE_GRAPHE.equals(cmd)) {
            jGraphe.setVisible(((JToggleButton) ae.getSource()).isSelected());
        }
    }

    //This one listens for edits that can be undone.
    protected class MyUndoableEditListener implements UndoableEditListener {
        public void undoableEditHappened(UndoableEditEvent e) {
            //Remember the edit and update the menus.
            undo.addEdit(e.getEdit());
            undoAction.updateUndoState();
            redoAction.updateRedoState();
        }
    }

    class UndoAction extends AbstractAction {
        public UndoAction() {
            super("Annuler");
            setEnabled(false);
        }

        public void actionPerformed(ActionEvent e) {
            try {
                undo.undo();
            } catch (CannotUndoException ex) {
                System.out.println("Unable to undo: " + ex);
                ex.printStackTrace();
            }

            updateUndoState();
            redoAction.updateRedoState();
        }

        protected void updateUndoState() {
            if (undo.canUndo()) {
                setEnabled(true);
                putValue(Action.NAME, undo.getUndoPresentationName());
            } else {
                setEnabled(false);
                putValue(Action.NAME, "Cancel");
            }
        }
    }

    class RedoAction extends AbstractAction {
        public RedoAction() {
            super("Repeat");
            setEnabled(false);
        }

        public void actionPerformed(ActionEvent e) {
            try {
                undo.redo();
            } catch (CannotRedoException ex) {
                System.out.println("Unable to redo: " + ex);
                ex.printStackTrace();
            }

            updateRedoState();
            undoAction.updateUndoState();
        }

        protected void updateRedoState() {
            if (undo.canRedo()) {
                setEnabled(true);
                putValue(Action.NAME, undo.getRedoPresentationName());
            } else {
                setEnabled(false);
                putValue(Action.NAME, "Repeat");
            }
        }
    }
}
