/*
 * Copyright 2005 Jean-Philippe Aumasson <jpnossa@free.fr>
 * Copyright 2005 Florent Kaisser <jpnossa@free.fr>
 * Copyright 2005 Jean-Baptiste Langlois <florent.kaisser@free.fr>
 *
 * This file is part of SIMS : SIMS Is a Machine Simulator
 *
 * SIMS is free software; you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation; either version 2 of the License, or
 * (at your option) any later version.
 *
 * SIMS is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with SIMS; if not, write to the Free Software
 * Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
 *
 * For more details see the file COPYING.
 */
/**
 * the gui to update tape content during the simulation
 *
 * @author JP <jpnossa@free.fr>
 * @version 20/12/04
 */
package sims.ihm.mdt;

import sims.*;

import sims.ihm.*;

import sims.mdt.*;

import java.awt.*;
import java.awt.event.*;
import java.awt.geom.*;

import java.io.*;

import java.util.*;

import javax.swing.*;
import javax.swing.event.*;


public class TapeEditor extends JFrame implements GraphicValues {
    private JTextField field;
    private JButton save;
    private JButton reset;
    private JButton quitter;
    private String backup;
    private Simulation sim;
    private int tape;

    ////////////////////////////////////////////////////////////////////

    /**
    * Initialize the tape editor, for the tape <tt>i</tt> of the simulation <tt>sim</tt>.
    *
    */
    public TapeEditor(Integer i, Simulation simm) {
        sim = simm;
        tape = i.intValue();
        backup = sim.wordOnTape(tape);
        setTitle("Bande " + i);
        setSize(TMODIF_WIDTH, TMODIF_HEIGH);
        setResizable(false);
        getContentPane().setLayout(new FlowLayout(FlowLayout.CENTER, 10, 10));
        field = new JTextField(30);
        field.setText(sim.wordOnTape(tape));

        save = new JButton("Save");
        reset = new JButton("Reset");
        quitter = new JButton("Quit");

        save.addActionListener(new ActionListener() {
                public void actionPerformed(ActionEvent e) {
                    saveTape();
                }
            });

        reset.addActionListener(new ActionListener() {
                public void actionPerformed(ActionEvent e) {
                    resetTape();
                }
            });

        quitter.addActionListener(new ActionListener() {
                public void actionPerformed(ActionEvent e) {
                    quit();
                }
            });

        getContentPane().add(field);
        getContentPane().add(save);
        getContentPane().add(reset);
        getContentPane().add(quitter);

        //place at the center of screen
        Dimension dim = Toolkit.getDefaultToolkit().getScreenSize();
        setLocation((dim.width / 2) - (getWidth() / 2),
            (dim.height / 2) - (getHeight() / 2));
        setVisible(true);
    }

    ////////////////////////////////////////////////////////////////////
    private void quit() {
        setVisible(false);
        dispose();

        try {
            finalize();
        } catch (Throwable t) {
        }
    }

    ////////////////////////////////////////////////////////////////////
    private void saveTape() {
        sim.setTapeContent(field.getText(), tape);
    }

    ////////////////////////////////////////////////////////////////////
    private void resetTape() {
        sim.setTapeContent(backup, tape);
        field.setText(backup);
    }
}
