/*
 * Copyright 2005 Jean-Philippe Aumasson <jpnossa@free.fr>
 * Copyright 2005 Florent Kaisser <jpnossa@free.fr>
 * Copyright 2005 Jean-Baptiste Langlois <florent.kaisser@free.fr>
 *
 * This file is part of SIMS : SIMS Is a Machine Simulator
 *
 * SIMS is free software; you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation; either version 2 of the License, or
 * (at your option) any later version.
 *
 * SIMS is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with SIMS; if not, write to the Free Software
 * Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
 *
 * For more details see the file COPYING.
 */
/**
 * the gui to update tape content during the simulation
 * @author JP
 * @version 07/12/04
 */
package sims.ihm.mdt;

import sims.*;

import sims.ihm.*;

import sims.mdt.*;

import java.awt.*;
import java.awt.event.*;
import java.awt.geom.*;

import java.io.*;

import java.util.*;

import javax.swing.*;
import javax.swing.event.*;


public class TapeUpdate extends JFrame implements GraphicValues {
    private static final String title = "Edition";
    private Simulation sim;
    private JLabel label;
    private JTextField buffer;
    private JComboBox box;

    public TapeUpdate(Simulation simm) {
        super();
        sim = simm;
        setTitle(title);
        setSize(TUPDT_WIDTH, TUPDT_HEIGHT);
        setResizable(false);
        getContentPane().setLayout(new FlowLayout(FlowLayout.CENTER, 10, 10));

        //placement au centre de l'cran
        Dimension dim = Toolkit.getDefaultToolkit().getScreenSize();
        setLocation((dim.width / 2) - (getWidth() / 2),
            (dim.height / 2) - (getHeight() / 2));

        WindowListener l = new WindowAdapter() {
                public void windowClosing(WindowEvent e) {
                    setVisible(false);
                    dispose();
                }
            };

        addWindowListener(l);

        label = new JLabel("Choose the tape to modify");
        label.setFont(POLICE);

        Vector values = new Vector();

        for (int i = 0; i < sim.getTapeNb(); i++)
            values.add(new Integer(i));

        box = new JComboBox(values);
        box.setSize(20, 50);

        JButton go = new JButton("Ok");

        go.addActionListener(new ActionListener() {
                public void actionPerformed(ActionEvent e) {
                    new TapeEditor((Integer) box.getSelectedItem(), sim);
                    quit();
                }
            });

        getContentPane().add(label);
        getContentPane().add(box);

        getContentPane().add(go);

        setVisible(true);
    }

    ////////////////////////////////////////////////////////////////////
    private void quit() {
        setVisible(false);
        sim = null;
        buffer = null;
        box = null;
        label = null;
        dispose();

        try {
            finalize();
        } catch (Throwable t) {
        }
    }
}
