/*
 * Copyright 2005 Jean-Philippe Aumasson <jpnossa@free.fr>
 * Copyright 2005 Florent Kaisser <jpnossa@free.fr>
 * Copyright 2005 Jean-Baptiste Langlois <florent.kaisser@free.fr>
 *
 * This file is part of SIMS : SIMS Is a Machine Simulator
 *
 * SIMS is free software; you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation; either version 2 of the License, or
 * (at your option) any later version.
 *
 * SIMS is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with SIMS; if not, write to the Free Software
 * Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
 *
 * For more details see the file COPYING.
 */
/**
 * a Turing machine visualisation of tapes.
 *
*  @author JP <jpnossa@free.fr>
*  @version 20/12/04
 */
package sims.ihm.mdt;

import sims.ihm.*;

import sims.mdt.*;

import java.awt.*;
import java.awt.event.*;

import javax.swing.*;


public class TuringView extends JFrame implements GraphicValues {
    private Simulation sim;
    private JScrollPane scroll;
    private String mtype; // machine type: determinist, ND, prob..
    private TmLauncher launcher;

    ////////////////////////////////////////////////////////////////////
    public TuringView(Simulation s, TmLauncher la) {
        sim = s;
        launcher = la;

        if (sim.isGravity()) {
            mtype = "Gravity";
        } else if (sim.isProb()) {
            mtype = "Probabilistic";
        } else if (sim.isND()) {
            mtype = "Non-deterministic";
        } else {
            mtype = "Deterministic";
        }

        WindowListener l = new WindowAdapter() {
                public void windowClosing(WindowEvent e) {
                    quit();
                }
            };

        addWindowListener(l);
        setSize(VIEW_WIDTH, VIEW_HEIGHT);

        setTitle("Visualisation: " + sim.getNom());
        setResizable(false);

        //placement 
        Dimension dim = Toolkit.getDefaultToolkit().getScreenSize();
        setLocation(5, dim.height - VIEW_HEIGHT - 20);

        JPan jp = new JPan();
        jp.setPreferredSize(new Dimension(5 * VIEW_WIDTH, VIEW_HEIGHT));

        scroll = new JScrollPane(jp, JScrollPane.VERTICAL_SCROLLBAR_AS_NEEDED,
                JScrollPane.HORIZONTAL_SCROLLBAR_ALWAYS);
        getContentPane().add(scroll);

        getContentPane().setBackground(Color.WHITE);

        setVisible(true);
    }

    ////////////////////////////////////////////////////////////////////

    /**
    * called when leaving the simulation
    */
    private void quit() {
        dispose();

        try {
            finalize();
        } catch (Throwable e) {
        }

        ;
        launcher.closeAll();
    }

    ////////////////////////////////////////////////////////////////////
    class JPan extends JPanel {
        protected JPan() {
            super(new FlowLayout());
        }

        public void paint(Graphics g) {
            super.paint(g);

            g.drawString("> Machine: " + sim.getNom(), 20, 15);
            g.drawString("> " + mtype + " > Current state: " +
                sim.currentState() + " > Time: " + sim.getTime() +
                " > Space: " + sim.getSpace(), 20, 30);

            setBackground(Color.WHITE);

            for (int j = 0; j < sim.nbTapes(); j++) { // 70 <- 50
                g.drawString("> Tape " + j + " :", 20, 50 + (j * TAPES_SPACE));

                // si on est trop  droite
                if ((sim.headPosition(j) - sim.min(j)) >= LAST_CELL) {
                    // case of scannedCell righter than the window limit
                    for (int i = 0; i < TAPE_SIZE; i++) {
                        if (i == ((sim.headPosition(j) - sim.min(j)) -
                                ((sim.headPosition(j) - sim.min(j)) -
                                LAST_CELL))) { // -sim.min(), si min<0, les indices sont dcals de |min|
                            g.setColor(Color.RED);
                            g.fillRect(i * CELL_SIZE, ((j) * TAPES_SPACE) + 60,
                                CELL_SIZE, CELL_SIZE);
                            g.setColor(Color.BLACK);
                        }

                        g.drawRect(i * CELL_SIZE, ((j) * TAPES_SPACE) + 60,
                            CELL_SIZE, CELL_SIZE);

                        // wordOnTape: retourne le mot du dbut (eventuelt ngatif, => min<0)...
                        // => wordOnTape[0] ~ wordOnTape[min], min pas forcment 0...
                        if (i < (sim.wordOnTape(j).length() -
                                ((sim.headPosition(j) - sim.min(j)) -
                                LAST_CELL))) {
                            g.drawString((new Character(sim.wordOnTape(j)
                                                           .charAt(i +
                                        ((sim.headPosition(j) - sim.min(j)) -
                                        LAST_CELL)))).toString(),
                                (i * CELL_SIZE) + 4, ((j) * TAPES_SPACE) + 73);
                        }
                    }
                } else {
                    // default case
                    for (int i = 0; i < TAPE_SIZE; i++) {
                        if (i == (sim.headPosition(j) - sim.min(j))) { // -sim.min(), si min<0, les indices sont dcals de |min|
                            g.setColor(Color.RED);
                            g.fillRect(i * CELL_SIZE, ((j) * TAPES_SPACE) + 60,
                                CELL_SIZE, CELL_SIZE);
                            g.setColor(Color.BLACK);
                        }

                        g.drawRect(i * CELL_SIZE, ((j) * TAPES_SPACE) + 60,
                            CELL_SIZE, CELL_SIZE);

                        // wordOnTape: retourne le mot du dbut (eventuelt ngatif, => min<0)...
                        // => wordOnTape[0] ~ wordOnTape[min], min pas forcment 0...
                        if (i < sim.wordOnTape(j).length()) {
                            g.drawString((new Character(sim.wordOnTape(j)
                                                           .charAt(i))).toString(),
                                (i * CELL_SIZE) + 4, ((j) * TAPES_SPACE) + 73);
                        }
                    }
                }
            }
        }
    }
}
