/*
 * Copyright 2005 Jean-Philippe Aumasson <jpnossa@free.fr>
 * Copyright 2005 Florent Kaisser <jpnossa@free.fr>
 * Copyright 2005 Jean-Baptiste Langlois <florent.kaisser@free.fr>
 *
 * This file is part of SIMS : SIMS Is a Machine Simulator
 *
 * SIMS is free software; you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation; either version 2 of the License, or
 * (at your option) any later version.
 *
 * SIMS is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with SIMS; if not, write to the Free Software
 * Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
 *
 * For more details see the file COPYING.
 */
//$Id: Graphe.java 1.5 2004/11/27 15:30:19 Florent Exp Florent $
package sims.mar;

import java.util.*;


/**
 *
 * Structure de donne Graphe
 * Un graphe  une ensemble de Sommet et d'Arete
 *
 * @author florent.kaisser@free.fr
 * @version $Id: Graphe.java 1.5 2004/11/27 15:30:19 Florent Exp Florent $
 *
 * $Log: Graphe.java $
 * Revision 1.5  2004/11/27 15:30:19  Florent
 * ajout de la mthode getSommet
 *
 * Revision 1.4  2004/11/02 18:01:12  Florent
 * Ajout de commentaires
 *
 * Revision 1.3  2004/11/01 20:56:04  Florent
 * Ajout de commentaire javadoc d'en-tete
 *
 */
public class Graphe {
    /**
    * Ensemble des sommets du graphe
    * @element-type Sommet
    */
    protected Vector mySommet = new Vector();

    /**
    * Ensemble des aretes du graphe
    * @element-type Arete
    */
    protected Vector myArete = new Vector();

    /**
     * Creer une collection des successeur d'un sommet
     * @param sommet Sommet dont on veut obtenir les successeurs
     * @param type Type d'arete qui relie les successers
     * @return La collection de succeseurs
     */
    public Collection getSuccesseurs(Sommet sommet, int type) {
        Vector ens = new Vector();
        Iterator i = myArete.iterator();

        while (i.hasNext()) {
            Arete arete = (Arete) i.next();

            if ((arete.getDebut() == sommet) && (arete.getType() == type)) {
                ens.add(arete.getFin());
            }
        }

        return ens;
    }

    /**
     *
     * @return l'ensemble des sommets du graphe
     */
    public Collection getSommets() {
        return mySommet;
    }

    /**
     *
     * Ajoute un sommet au graphe
     * @param sommet Sommet  ajouter
     */
    public void ajouterSommet(Sommet sommet) {
        mySommet.add(sommet);

        //System.out.println("Ajout du sommet : "+sommet);
    }

    /**
     * supprime tous les sommets et tous les aretes
     */
    public void clear() {
        mySommet.clear();
        myArete.clear();
    }

    /**
     * Ajoute une arete au graphe
     * @param arete Arete  ajouter
     */
    public void ajouterArete(Arete arete) {
        myArete.add(arete);

        //System.out.println("Ajout de l'arete : "+arete.getDebut()+" -> "+arete.getFin()+ " " + arete.getType()); 	
    }
}
