/*
 * Copyright 2005 Jean-Philippe Aumasson <jpnossa@free.fr>
 * Copyright 2005 Florent Kaisser <jpnossa@free.fr>
 * Copyright 2005 Jean-Baptiste Langlois <florent.kaisser@free.fr>
 *
 * This file is part of SIMS : SIMS Is a Machine Simulator
 *
 * SIMS is free software; you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation; either version 2 of the License, or
 * (at your option) any later version.
 *
 * SIMS is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with SIMS; if not, write to the Free Software
 * Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
 *
 * For more details see the file COPYING.
 */
//$Id: Instruction.java 1.6 2004/11/27 15:30:19 Florent Exp Florent $
package sims.mar;


/**
 *
 * Une instruction est un sommet executable
 * qui est toujours associ  une commande
 *
 * @author florent.kaisser@free.fr
 * @version $Id: Instruction.java 1.6 2004/11/27 15:30:19 Florent Exp Florent $
 *
 * $Log: Instruction.java $
 * Revision 1.6  2004/11/27 15:30:19  Florent
 * association de l'instruction  une ligne du code source
 * ajout des methode getLigne,setLigne, et du membre ligne
 *
 * Revision 1.5  2004/11/02 18:01:12  Florent
 * Ajout de commentaires
 *
 * Revision 1.4  2004/11/01 20:56:04  Florent
 * Ajout de commentaire javadoc d'en-tete
 * Numrotation des instruction
 *
 */
public class Instruction extends Sommet {
    //commande associ  l'intruction
    private Commande myCommande;
    private int id; //identificateur de l'instruction
    private int ligne; //ligne correpondante  cette instruction, pour le debeugage

    /**
     *
     * @param commande Commande  associer  l'instruction
     */
    Instruction(Commande commande, int id) {
        //Si la commande est fourni initialise l'intitul
        super((commande == null) ? "" : commande.toString());
        myCommande = commande;

        //initialise l'id de l'instruction
        this.id = id;
        ligne = -1; //aucune ligne ne correpond  cette instruction
    }

    /**
     * Execute l'intruction
     * @return le type de branchement
     */
    public int executer() {
        return myCommande.executer();
    }

    /**
     *
     * @return l'id de l'intruction
     */
    public int getID() {
        return id;
    }

    /**
     * Si la ligne vaut -1, aucune ligne n'est associ  cette exception
     * @return la ligne correspondant  l'instruction
     */
    public int getLigne() {
        return ligne;
    }

    /**
     * @param ligne Numro de ligne  associer avec l'intruction
     */
    public void setLigne(int ligne) {
        this.ligne = ligne;
    }

    /**
     *
     * Associ la commande  l'intruction
     * @param cmd Commade  associer
     */
    public void setCommande(Commande cmd) {
        myCommande = cmd;
        intitule = cmd.toString();
    }

    /**
     *
     * @return la commande associ  l'instruction
     */
    public Commande getCommande() {
        return myCommande;
    }
}
