//$Id: MachineARegistre.java 1.6 2004/11/28 21:43:51 Florent Exp Florent $
package sims.mar;

import sims.*;

import java.io.*;

import java.lang.String;

import java.util.*;


/**
 *
 * Control le programme de la machine  registre (Simulateur)
 * Appeler la methode start pour demarrer la machine dans un nouveau thread
 *
 * @author florent.kaisser@free.fr
 * @version $Id: MachineARegistre.java 1.6 2004/11/28 21:43:51 Florent Exp Florent $
 *
 * $Log: MachineARegistre.java $
 * Revision 1.6  2004/11/28 21:43:51  Florent
 * Ajout de l'exception Erreur de compilation
 *
 * Revision 1.5  2004/11/27 15:30:19  Florent
 * adaptation  l'interface InterfaceSimulation
 * meilleur syncrhonisation du thread
 *
 * Revision 1.4  2004/11/02 18:01:12  Florent
 * Ajout de commentaires
 *
 * Revision 1.3  2004/11/01 20:56:04  Florent
 * ajout des point d'arrets
 * vitesse
 * nom de programme
 * thread
 *
 */
public class MachineARegistre extends Thread implements InterfaceSimulation {
    /* {src_lang=Java}*/

    //chemin du fcihier de la machine  registre courante
    private File fichierCourant;
    public Programme myProgramme = null;
    private int vitesse = 5;

    //Ensemble d'instruction o le programme s'arrete
    private Collection pointArret;
    private String nom;
    private boolean pointArretActif = false;
    private boolean programmeTermine = false;

    //correspond au chagement d'etat : continue/pas  pas
    private boolean threadSuspended = true;
    private boolean threadStop = false;
    ActionMAR actionMAR = null; //objet contenant la mthode qu'il faut appel lors d'une execution d'instruction

    /**
     * @return le nom de la machine
     */
    public String getNom() {
        return nom;
    }

    /**
      * @return la vitesse de la machine
     */
    public int getVitesse() {
        return vitesse;
    }

    /**
     *
     * @return les point d'arret (Collection d'Instruction)
     */
    public Collection getPointArret() {
        return pointArret;
    }

    /**
     *
     * @return le programme associ  la machine
     */
    public Programme getProgramme() {
        return myProgramme;
    }

    /**
     * Lance la machine
     * Pour savoir quand l'execution est termin utiliser @see #attendreLaFin()
     * La machine s'arrete si elle arrive sur un point d'arret
     * elle repare quand on appel la methode reprendre
     **/
    public void run() {
        try {
            while (!threadStop) {
                //attend une certaine periode pour controler la vitesse
                if (vitesse > 0) {
                    sleep(InterfaceSimulation.VITESSES[vitesse]);
                }

                //Si on arrive sur un point d'arret on s'arret	
                if (pointArretActif) {
                    if (pointArret.contains(
                                myProgramme.getInstructionCourante())) {
                        threadSuspended = true;
                        actionMAR.programmeBreakPoint();
                    }
                }

                synchronized (this) {
                    while (threadSuspended)
                        wait();
                }

                //Execute l'instruction	
                prochainPas();
            }

            threadStop = false;
        } catch (InterruptedException ie) {
        }
    }

    /**
     *
     * @return
     */
    public void setAction(ActionMAR action) {
        actionMAR = action;
    }

    synchronized public void decharger() {
        myProgramme = null;
        fichierCourant = null;
        programmeTermine = false;
        arreter();
        myProgramme = null;
    }

    /**
     * Charge un fichier
     * @param path chemin du fichier
     */
    synchronized public void charger(File file)
        throws IOException, ExceptionCompilation {
        myProgramme = null;
        fichierCourant = file;
        programmeTermine = false;
        arreter();

        ParseFileREG pfm = new ParseFileREG(new FileInputStream(file));

        //parse le fichier
        myProgramme = pfm.parse();

        //se place  la premiere intruction (debut)	
        myProgramme.allerPremiereInstruction();

        //initialise les variable de classe
        pointArret = pfm.getPointArret();
        pointArretActif = (pointArret.size() > 0);
        vitesse = pfm.getVitesse();
        nom = pfm.getNom();

        if (actionMAR != null) {
            actionMAR.programmeDebut();
        }
    }

    synchronized public void charger(String fileName)
        throws IOException, ExceptionCompilation {
        charger(new File(fileName));
    }

    public void tuer() {
        threadStop = true;
        threadSuspended = false;
    }

    public boolean estArrete() {
        return threadSuspended;
    }

    synchronized public void demarrer() {
        if (!isAlive()) {
            start();
        }

        if (threadSuspended) {
            threadSuspended = false;
        }

        notifyAll();
    }

    synchronized public void arreter() {
        if (!threadSuspended) {
            threadSuspended = true;
        }
    }

    public void reinitialiser() {
        try {
            int oldVitesse = vitesse;
            charger(fichierCourant);
            vitesse = oldVitesse;
        } catch (ExceptionCompilation e) {
        } catch (IOException ioe) {
        }
    }

    public void prochainPas() {
        try {
            Instruction i = myProgramme.getInstructionCourante();
            myProgramme.prochaineInstruction();

            if (actionMAR != null) {
                actionMAR.instructionExecute(i);
            }
        } catch (ExceptionPasDeSuccesseur e) {
            programmeTermine = true;
            arreter();

            //programme termin
            //c la fin
            if (actionMAR != null) {
                actionMAR.programmeFin();
            }
        }
    }

    public void incVitesse() {
        if (vitesse < 5) {
            vitesse++;
        }

        if (actionMAR != null) {
            actionMAR.vitesseChange(vitesse);
        }
    }

    public void decVitesse() {
        if (vitesse > 1) {
            vitesse--;
        }

        if (actionMAR != null) {
            actionMAR.vitesseChange(vitesse);
        }
    }

    /**
     *
     * Attend la fin de l'execution de la machine
     * @throws InterruptedException
     */
    synchronized public void attendreLaFin() throws InterruptedException {
        while (!programmeTermine)
            wait();
    }

    /* public void instructionExecute(Instruction i,Object o){
          System.out.println(i.getLigne()+ " " + i.getCommande());
    }
    public void programmeFin(Object o){
            System.out.println("Fin du programme");
    }

    static public void main(String arg[])  throws IOException,InterruptedException{
          MachineARegistre m;
          m=new MachineARegistre(arg[0]);
          m.setAction(m);

          m.demarrer();

          m.attendreLaFin();



          System.out.println(m.getNom());
          System.out.println(m.getProgramme().getRegistres());


    }*/
}
