/*
 * Copyright 2005 Jean-Philippe Aumasson <jpnossa@free.fr>
 * Copyright 2005 Florent Kaisser <jpnossa@free.fr>
 * Copyright 2005 Jean-Baptiste Langlois <florent.kaisser@free.fr>
 *
 * This file is part of SIMS : SIMS Is a Machine Simulator
 *
 * SIMS is free software; you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation; either version 2 of the License, or
 * (at your option) any later version.
 *
 * SIMS is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with SIMS; if not, write to the Free Software
 * Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
 *
 * For more details see the file COPYING.
 */
//$Id: ParseFileREG.java 1.5 2004/11/28 21:43:51 Florent Exp Florent $
package sims.mar;

import sims.*;

import java.io.*;

import java.util.*;


/**
 *
 * Lis un fichier REG qui permet de charger un programme
 *
 * @author florent.kaisser@free.fr
 * @version $Id: ParseFileREG.java 1.5 2004/11/28 21:43:51 Florent Exp Florent $
 *
 * $Log: ParseFileREG.java $
 * Revision 1.5  2004/11/28 21:43:51  Florent
 * Ajout de l'exception erreur de compilation
 * BUG : un label vide doit etre incorecte
 *
 * Revision 1.4  2004/11/27 15:30:19  Florent
 * Aucune modification
 *
 * Revision 1.3  2004/11/02 18:01:12  Florent
 * Ajout de commentaires
 *
 * Revision 1.2  2004/11/01 20:56:04  Florent
 * gestion des commentaires
 * finition des directives
 *
 */
public class ParseFileREG {
    private int c;
    private Programme prog;
    private Instruction instructionFin;
    private FileInputStream f;
    private Hashtable label = new Hashtable();

    //contien tous les labels o on s'arrete (directive stop)
    private Set nom_stops = new HashSet();
    private Set inst_stops = new HashSet();
    private int ligne = 0;
    String nom = new String("Register machine");
    int vitesse = 0;
    private int idInstSuivant = 0;

    /**
     *
     * @param f :  descripteur de fichier sur lequel on va le parser
     * @throws IOException
     */
    ParseFileREG(FileInputStream f)
        throws IOException, IOException, ExceptionCompilation {
        this.f = f;
        c_suiv();
    }

    /**
     * @return le nom de la machine
     */
    public String getNom() {
        return nom;
    }

    /**
     * @return la vitesse d'execution @see sims.mar.Programme#getVitesse()
     */
    public int getVitesse() {
        return vitesse;
    }

    /**
     * @return  les point d'arret @see sims.mar.Programme#getPointArret()
     */
    public Collection getPointArret() {
        return (Collection) inst_stops;
    }

    private void initPointArret() {
        for (Iterator i = nom_stops.iterator(); i.hasNext();)
            inst_stops.add((Instruction) label.get(i.next()));
    }

    //Cre le registre et l'intruction Fin
    private void AjouterFin() {
        Fin fin = new Fin();
        instructionFin = createInstruction("fin", fin);
    }

    private Registre AjouterRegistre(int id, int val) {
        Registre r = new Registre(val, id);

        prog.registres.put(new Integer(id), r);

        return r;
    }

    //retourn le registre qui a l'identificateur id,
    //si il n'existe pas, le cre
    private Registre getRegistre(int id) {
        Registre registre = (Registre) prog.registres.get(new Integer(id));

        if (registre == null) {
            registre = AjouterRegistre(id, 0);
        }

        return registre;
    }

    //Cre le registre debut
    private void AjouterDebut() throws ExceptionCompilation {
        Instruction inst = (Instruction) label.get("debut");
        Debut debut = new Debut();
        Instruction instructionDebut = createInstruction(debut);

        if ((inst == null) || ((inst != null) && (inst.getCommande() == null))) {
            erreur("Label 'debut' not defined");
        }

        prog.ajouterArete(new Arete(1, instructionDebut, inst));
        prog.debut = instructionDebut;
    }

    //Cre une instruction et l'ajoute au programme
    private Instruction createInstruction(String id, Commande cmd) {
        Instruction instruction = new Instruction(cmd, idInstSuivant++);
        label.put(id, instruction);
        prog.ajouterSommet(instruction);

        return instruction;
    }

    private Instruction createInstruction() {
        return createInstruction(null);
    }

    private Instruction createInstruction(Commande cmd) {
        Instruction instruction = new Instruction(cmd, idInstSuivant++);
        prog.ajouterSommet(instruction);

        return instruction;
    }

    //Cherche l'instruction correponbdant  l'id, la cre si elle exitse pas
    private Instruction getInstruction(String id, Commande cmd)
        throws IOException {
        Instruction instruction = (Instruction) label.get(id);

        if (instruction == null) {
            instruction = createInstruction(id, cmd);
        }

        if (cmd != null) {
            instruction.setCommande(cmd);
        }

        //throw new IOException("Identificateur introuvable  la ligne " + ligne);
        return instruction;
    }

    private Instruction getInstruction(String id)
        throws IOException, ExceptionCompilation {
        return getInstruction(id, null);
    }

    private void readCaractere() throws IOException, ExceptionCompilation {
        int old_c = c;

        if (c == '\n') {
            ligne++;
        }

        c = f.read();

        if ((c == -1) && (old_c == -1)) {
            erreur("end file non awaited");
        }
    }

    private void c_suiv() throws IOException, ExceptionCompilation {
        readCaractere();

        //gestion des commentaires
        if (((char) c) == '/') {
            readCaractere();

            switch (c) {
            case '/':

                int new_ligne;

                do {
                    new_ligne = ligne;
                    readCaractere();
                } while (ligne == new_ligne);

                break;

            case '*':

                do {
                    readCaractere();

                    if (c == '*') {
                        readCaractere();

                        if (c == '/') {
                            readCaractere();

                            break;
                        }
                    }
                } while (c != -1);

                break;
            }
        }
    }

    private void erreur(String msg) throws ExceptionCompilation {
        throw new ExceptionCompilation(ligne, msg);
    }

    private void erreur() throws ExceptionCompilation {
        erreur("");
    }

    private boolean estEspace(int c) {
        return ((c == ' ') || (c == '\t') || (c == '\r') || (c == '\n') ||
        (c == ','));
    }

    private void lireEspace() throws IOException, ExceptionCompilation {
        while (estEspace(c))
            c_suiv();
    }

    private int lireEntier() throws IOException, ExceptionCompilation {
        String entier = "";

        while ((c >= '0') && (c <= '9')) {
            entier += ((char) c);
            c_suiv();
        }

        if (entier.length() == 0) {
            erreur("Entier attendu");
        }

        return Integer.parseInt(entier);
    }

    //chaine de caractre etre guillemet
    private String lireString() throws IOException, ExceptionCompilation {
        String str = new String();

        if (c != '"') {
            erreur("String must be surrounded by double quotes");
        }

        c_suiv();

        //on lit jusqu'a se qu'on trouve un autre guillemet	
        while (c != '"') {
            str += ((char) c);
            readCaractere(); //on n'utilise pas c_suiv, pour passer le traitement des commentaires
        }

        c_suiv();

        return str;
    }

    private String lireLabel() throws IOException, ExceptionCompilation {
        String id = new String();

        if ((c >= '0') && (c <= '9')) {
            erreur("Bad label");
        }

        while (((c >= 'A') && (c <= 'Z')) || ((c >= 'a') && (c <= 'z')) ||
                ((c >= '0') && (c <= '9'))) {
            id += ((char) c);
            c_suiv();
        }

        if (id.length() == 0) {
            erreur("Bad label");
        }

        return id;
    }

    private void passerJusquaEgal() throws IOException, ExceptionCompilation {
        //on lit jusqu'a se qu'on trouve un =
        while ((c != '=') && (c >= 0))
            c_suiv();

        if (c != '=') {
            erreur("= awaited");
        }

        c_suiv();

        lireEspace();
    }

    //lis des labels spar pas des virgules, et les ajoute a la collection des stops
    private void lireLabelsStop() throws IOException, ExceptionCompilation {
        while (true) {
            lireEspace();

            String labelo = lireLabel();
            nom_stops.add(labelo);
            lireEspace();

            if (c != ',') {
                break;
            }

            c_suiv();

            //on repete jusqu'a qu'il y est plus de virgule			
        }
    }

    //lit la directive et execute l'action appropri
    private void lireDirective() throws IOException, ExceptionCompilation {
        if (c != '#') {
            erreur("# awaited");
        }

        c_suiv();

        switch (c) {
        case 'R':
            c_suiv();

            int num = lireEntier();
            passerJusquaEgal();

            int val = lireEntier();
            AjouterRegistre(num, val);

            break;

        case 'v':
            passerJusquaEgal();
            vitesse = lireEntier();

            break;

        case 's':
            passerJusquaEgal();
            lireLabelsStop();

            break;

        case 'n':
            passerJusquaEgal();
            nom = lireString();

            break;
        }
    }

    private void lireDirectives() throws IOException, ExceptionCompilation {
        lireEspace();

        while (c == '#') {
            lireDirective();
            lireEspace();
        }
    }

    private Instruction lireInstruction()
        throws IOException, ExceptionCompilation {
        String labele = lireLabel();
        Instruction inst = null;
        lireEspace();

        //si le label est ni inc ni dec, alors c pas une commande
        if ((labele.compareTo("dec") != 0) && (labele.compareTo("inc") != 0)) {
            inst = getInstruction(labele);

            //si c est == : ce une definition d'intruction, on l'initialise ci dessous
            if (c == ':') {
                c_suiv();
                lireEspace();

                //si c'est la dfinition du label fin, on l'ignore
                if (inst == instructionFin) {
                    return null;
                }

                //lis le nom de la commande
                labele = lireLabel();
                lireEspace();
            } else {
                //le cas o c'est un saut
                return inst;
            }
        } else {
            //le cas o le label est omis dans un saut
            inst = createInstruction();
        }

        inst.setLigne(ligne);

        Commande cmd;
        int idRegistre = lireEntier();
        lireEspace();

        if (labele.compareTo("dec") == 0) {
            cmd = new RegistreMoins(getRegistre(idRegistre));

            Instruction zero = lireInstruction();
            Instruction succ = lireInstruction();
            prog.ajouterArete(new Arete(0, inst, zero));
            prog.ajouterArete(new Arete(1, inst, succ));
            inst.setCommande(cmd);
        } else if (labele.compareTo("inc") == 0) {
            cmd = new RegistrePlus(getRegistre(idRegistre));

            Instruction succ = lireInstruction();
            prog.ajouterArete(new Arete(1, inst, succ));
            inst.setCommande(cmd);
        }
        //si c les sommet fin, on a rien  faire
        else if (labele.compareTo("fin") == 0) {
            ;
        } else {
            erreur("Command 'inc' or 'dec' awaited");
        }

        return inst;
    }

    private void lireInstructions() throws IOException, ExceptionCompilation {
        lireEspace();

        while (c != -1) {
            lireInstruction();
            lireEspace();
        }
    }

    protected void verifierCommandes() throws IOException, ExceptionCompilation {
        Collection inst = prog.getSommets();

        for (Iterator i = inst.iterator(); i.hasNext();)
            if (((Instruction) i.next()).getCommande() == null) {
                erreur("Instruction with no command associated");
            }
    }

    /**
     *
     * Parse le fichier
     * @return le Programme lu dans le fichier
     * @throws IOException Erreur de lecture ou de syntaxe
     */
    public Programme parse() throws IOException, ExceptionCompilation {
        prog = new Programme();
        AjouterFin();

        lireDirectives();
        lireInstructions();

        AjouterDebut();
        initPointArret();
        verifierCommandes();

        return prog;
    }
}
