/*
 * Copyright 2005 Jean-Philippe Aumasson <jpnossa@free.fr>
 * Copyright 2005 Florent Kaisser <jpnossa@free.fr>
 * Copyright 2005 Jean-Baptiste Langlois <florent.kaisser@free.fr>
 *
 * This file is part of SIMS : SIMS Is a Machine Simulator
 *
 * SIMS is free software; you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation; either version 2 of the License, or
 * (at your option) any later version.
 *
 * SIMS is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with SIMS; if not, write to the Free Software
 * Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
 *
 * For more details see the file COPYING.
 */
//$Id: Programme.java 1.6 2004/11/28 21:43:51 Florent Exp Florent $
package sims.mar;

import sims.*;

import sims.ihm.graph.*;

import java.util.*;


/**
 *
 * Programme de la machine  registre
 *
 * @author florent.kaisser@free.fr
 * @version $Id: Programme.java 1.6 2004/11/28 21:43:51 Florent Exp Florent $
 *
 * $Log: Programme.java $
 * Revision 1.6  2004/11/28 21:43:51  Florent
 * Ajout de l'exception Pas de successeur
 *
 * Revision 1.5  2004/11/27 15:30:19  Florent
 * ajout des methodes getRegistre et getNbrRegistre
 *
 * Revision 1.4  2004/11/02 18:01:12  Florent
 * Ajout de commentaires
 *
 * Revision 1.3  2004/11/01 20:56:04  Florent
 * optimisation en ajoutant une matrice de succeseurs
 *
 */
public class Programme extends Graphe implements GrapheInterface {
    Instruction debut; //instruction correspondant au sommet "debut"

    //ensemble contenat les registres, index par son indice
    Hashtable registres = new Hashtable();
    private Instruction instructionCourante;

    /*Matrice permetant d'optimisser la recherche de successeur*/
    private Instruction[][] matrice;

    private void createMatrice() {
        Iterator i = myArete.iterator();
        matrice = new Instruction[mySommet.size()][2];

        while (i.hasNext()) {
            Arete arete = (Arete) i.next();
            matrice[((Instruction) arete.getDebut()).getID()][arete.getType()] = (Instruction) arete.getFin();
        }
    }

    /**
     * @return la prochaine instruction (Debut)
     */
    public Instruction getPremiereInstruction() {
        return debut;
    }

    /**
     * Executre l'intruction courante, est saute  la prochaine instruction
     * @throws Exception Lorsque la fin du programme est atteinte
     */
    public void prochaineInstruction() throws ExceptionPasDeSuccesseur {
        //execute l'instruction courante
        int type = instructionCourante.executer();

        //System.out.println(getInstructionCourante() + " " + getInstructionCourante().getID());
        //recupere le succeseur du type correspondant
        Instruction inst = matrice[instructionCourante.getID()][type];

        if (inst == null) {
            //si on a pas trouv de succeseur, on lve une execption
            throw new ExceptionPasDeSuccesseur();
        } else {
            instructionCourante = inst;
        }
    }

    /**
     *
     * @return l'intruction courante (la prochiane  executer)
     */
    public Instruction getInstructionCourante() {
        return instructionCourante;
    }

    /**
     *
     * @return Ensemble des registres du programme
     */
    public Collection getRegistres() {
        return (Collection) registres.values();
    }

    public Registre getRegistre(int indice) {
        return (Registre) registres.get(new Integer(indice));
    }

    public int getNbrRegistre() {
        return registres.size();
    }

    /**
     * Saute  la preimer instruction (debut)
     *
     */
    public void allerPremiereInstruction() {
        createMatrice();
        instructionCourante = debut;

        try {
            //exectuer l'etat debut
            prochaineInstruction();
        } catch (ExceptionPasDeSuccesseur e) {
            //ne sera jamais executer, le debut ayant toujours un succeseur
        }
    }

    /**
     *
     *         inplementation de l'interface GrapheInterface pour l'affichage du graphe
     */
    public Collection arcs() {
        int size;
        Vector c = new Vector(size = myArete.size());

        for (int i = 0; i < size; i++) {
            Arete a = (Arete) myArete.get(i);
            c.add(new Arc(a.getDebut(), a.getFin(),
                    (a.getType() == 0) ? "0" : ""));
        }

        return c;
    }

    public int getNbrEtat() {
        return mySommet.size();
    }

    public int getNbrArc() {
        return myArete.size();
    }

    public Object getRacine() {
        return debut;
    }
}
