/*
 * Copyright 2005 Jean-Philippe Aumasson <jpnossa@free.fr>
 * Copyright 2005 Florent Kaisser <jpnossa@free.fr>
 * Copyright 2005 Jean-Baptiste Langlois <florent.kaisser@free.fr>
 *
 * This file is part of SIMS : SIMS Is a Machine Simulator
 *
 * SIMS is free software; you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation; either version 2 of the License, or
 * (at your option) any later version.
 *
 * SIMS is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with SIMS; if not, write to the Free Software
 * Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
 *
 * For more details see the file COPYING.
 */
//$Id: Registre.java 1.6 2004/11/27 15:30:19 Florent Exp Florent $
package sims.mar;


/**
 *
 * Un registre est une valeur entiere qui peut etre
 * increment et decrement
 *
 * @author florent.kaisser@free.fr
 * @version $Id: Registre.java 1.6 2004/11/27 15:30:19 Florent Exp Florent $
 *
 * $Log: Registre.java $
 * Revision 1.6  2004/11/27 15:30:19  Florent
 * ajout de la methode intitule
 * modification de toString
 *
 * Revision 1.5  2004/11/02 18:01:12  Florent
 * Ajout de commentaires
 *
 * Revision 1.4  2004/11/01 20:56:04  Florent
 * ajout de commentaire d'en-tete javadoc
 *
 */
public class Registre {
    /* {src_lang=Java}*/
    private int valeur;
    private int indice;
    private int maxValeur = 0;
    private int nbrDec = 0;
    private int nbrInc = 0;

    /**
     *
     *
     * @param valeur valeur initiale du registre
     * @param indice indice du registre
     */
    Registre(int valeur, int indice) {
        this.valeur = valeur;
        this.indice = indice;

        calculMax();
    }

    public int getNbrDec() {
        return nbrDec;
    }

    public int getNbrInc() {
        return nbrInc;
    }

    public int getMaxValeur() {
        return maxValeur;
    }

    private void calculMax() {
        if (valeur > maxValeur) {
            maxValeur = valeur;
        }
    }

    /**
     *
     * @return la valeur du registre courant
     */
    public int getValeur() {
        return this.valeur;
    }

    /**
     * @param i nouvelle valeur du registre
     */
    public void setValeur(int i) {
        valeur = i;
        calculMax();
    }

    /**
     *
     *
     * @return l'indice du registre
     */
    public int getIndice() {
        return this.indice;
    }

    /**
     * incremente le registre
     */
    void inc() {
        valeur++;
        nbrInc++;
        calculMax();
    }

    /**
     * decremente le registre
     *
     * @return          1 si le registre est decrement
     *                         0 si le registre est egal  0 avant la decrementation
     */
    int dec() {
        nbrDec++;

        if (valeur > 0) {
            valeur--;

            return 1;
        }

        return 0;
    }

    public String intitule() {
        return "R" + indice;
    }

    public String toString() {
        return (new Integer(valeur)).toString();
    }
}
