/*
 * Copyright 2005 Jean-Philippe Aumasson <jpnossa@free.fr>
 * Copyright 2005 Florent Kaisser <jpnossa@free.fr>
 * Copyright 2005 Jean-Baptiste Langlois <florent.kaisser@free.fr>
 *
 * This file is part of SIMS : SIMS Is a Machine Simulator
 *
 * SIMS is free software; you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation; either version 2 of the License, or
 * (at your option) any later version.
 *
 * SIMS is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with SIMS; if not, write to the Free Software
 * Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
 *
 * For more details see the file COPYING.
 */
/**
* a turing machine instruction
* @author JP <jpnossa@free.fr>
* @version 20/12/04
*/
package sims.mdt;

import java.util.*;


public class Instruction {
    private int line; // this instruction line in the file
    private String newState; // the new state of this instruction
    private HashMap tapesTrans; // the tapes moves for this instruction

    ////////////////////////////////////////////////////////////////////

    /**
    * Instance a new instruction
    * @param l the line number in the file.
    * @param st the new state.
    */
    public Instruction(int l, String st) {
        line = l;

        newState = st;
        tapesTrans = new HashMap();
    }

    ////////////////////////////////////////////////////////////////////

    /**
    * Insert the move <tt>value</tt> as a move of this instruction for the tape <tt>tape</tt>.
    *
    */
    public void setTrans(int tape, String value) {
        tapesTrans.put(new Integer(tape), value);
    }

    ////////////////////////////////////////////////////////////////////

    /**
    *
    * @return the new state of this instruction.
    */
    public String getNewState() {
        return newState;
    }

    ////////////////////////////////////////////////////////////////////

    /**
    *
    * @return the move of the tape <tt>i</tt>.
    */
    public String getTapeTrans(int i) {
        return (String) tapesTrans.get(new Integer(i));
    }

    ////////////////////////////////////////////////////////////////////

    /**
    *
    * @return the line of this instruction.
    */
    public int getLine() {
        return line;
    }
}
