/*
 * Copyright 2005 Jean-Philippe Aumasson <jpnossa@free.fr>
 * Copyright 2005 Florent Kaisser <jpnossa@free.fr>
 * Copyright 2005 Jean-Baptiste Langlois <florent.kaisser@free.fr>
 *
 * This file is part of SIMS : SIMS Is a Machine Simulator
 *
 * SIMS is free software; you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation; either version 2 of the License, or
 * (at your option) any later version.
 *
 * SIMS is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with SIMS; if not, write to the Free Software
 * Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
 *
 * For more details see the file COPYING.
 */
/**
 * a Turing Machine read from a file, create Parser to read it...
 *
*  @author JP <jpnossa@free.fr>
*  @version 20/11/04
 */
package sims.mdt;

import sims.*;

import sims.ihm.*;

import sims.mdt.nd.*;

import java.io.*;

import java.util.*;


public class MachineRead extends TuringMachine {
    ////////////////////////////////////////////////////////////////////

    /**
     * class constructor, open file, and read it.
     * @param myFile the entry word
     *
     * */
    public MachineRead(String myFile) throws IOException, ExceptionCompilation {
        super();

        try {
            p = new TmParser(myFile, this);
        } catch (ExceptionCompilation e) {
            throw e;
        }

        if (inWord == null) {
            inWord = " ";
        }

        if (wordCorrect(inWord)) {
            if (choiceMachine) { // if ND
                rubans.add(new Tape(inWord, p.getAlphabet(), alpha)); // create main tape wit inWord

                for (int k = 1; k < nbTapes; k++) // create other tapes

                    rubans.add(new Tape(k, p.getAlphabet(), alpha));

                ndQueue = new Vector();
                ndMoveState = new HashMap();
                ndMachine = new NDMachine(this);

                if (gMachine) {
                    selectionner = new Vector();
                    weights = new HashMap();
                }

                //enqueueCurrentConf();
            } else { // Deterministic
                rubans.add(new Tape(inWord, p.getAlphabet(), alpha));

                for (int k = 1; k < nbTapes; k++)
                    rubans.add(new Tape(k, p.getAlphabet(), alpha));
            }

            mCopyTrans(); // copy transitions, in machine's and each tape's HashMaps 
        } else {
            TuringException.error("mot incorrect");
        }
    }

    ////////////////////////////////////////////////////////////////////

    /**
         * !!! CRITICAL METHOD !!!
         * copy instructions to the transition mappings
         *
         */
    protected void mCopyTrans() {
        String key;

        for (int i = 0; i < p.getNbInstructions(); i++) // for each instruction line
         {
            if ((!fullSized(i)) && (nbTapes > 1)) {
                specialSizedInst(i);
            } else { // if normally sized instruction
                key = createKey(i);

                if (!key.equals("np")) { // if no '*' included

                    if (nbTapes == 1) {
                        insertOneTapeTransitions(key, i);
                    } else { // several tapes
                        insertSeveralTapesTransitions(key, i);
                    }
                }

                // same key than for transtate...ouf, value: letter * shift <=> 4*i+3 : 4*i+4
                //key.delete(0, key.length()); // raz the key...
                key = null;
            }
        }
    }

    ////////////////////////////////////////////////////////////////////
    private void insertOneTapeTransitions(String key, int i) {
        Instruction myInst;

        if (moveState.containsKey(key.toString())) { // if key already here => ND
            setChoiceMachine();

            // if the key's not already mapped as an ND key
            if (!ndMoveState.containsKey(key.toString())) { // key not yet mapped

                // on cre notre instruction qu'on mettra dans le vecteur
                ndMoveState.put(key.toString(), new Vector());
            }

            myInst = new Instruction(p.getLine(i), p.getInstComp(i, 4));

            // IF key exists on ajoute juste this mapping as collision ds le hash nd
            ((Vector) ndMoveState.get(key.toString())).add(myInst);

            if (p.getInstComp(i, 2).equals("*")) // si '*' au char crit, on avait forcement * au lu...=> on ecrit le meme que le lu
             {
                myInst.setTrans(0,
                    (String) (key.toString().split("[.]")[1]) + "." +
                    p.getInstComp(i, 3));
            } else {
                myInst.setTrans(0,
                    (String) (p.getInstComp(i, 2) + "." + p.getInstComp(i, 3)));
            }
        } else {
            // FOR TRANSTATE: state * alpha^nbTapes -> state
            myInst = new Instruction(p.getLine(i), p.getInstComp(i, 4));

            moveState.put(key.toString(), myInst); // 4*nbTapes+1 = indice of last component = newState

            // put the transition string into our newly created Instruction
            myInst.setTrans(0,
                (String) (p.getInstComp(i, 2) + "." + p.getInstComp(i, 3)));
        }
    }

    ////////////////////////////////////////////////////////////////////
    private void insertSeveralTapesTransitions(String key, int i) {
        Instruction myInst;

        // FOR TRANSTATE: state * alpha^nbTapes -> state

        /* ************ ND HERE *************** */

        // if the key's already present => ND
        if (moveState.containsKey(key.toString())) {
            setChoiceMachine();

            // if the key's already mapped as an ND key
            if (!ndMoveState.containsKey(key.toString())) { // key not yet mapped
                ndMoveState.put(key.toString(), new Vector());
            }

            myInst = new Instruction(p.getLine(i),
                    (String) p.getInstComp(i, (4 * nbTapes) + 1));

            // IF key exists on ajoute juste this mapping co collision ds le hash nd
            ((Vector) ndMoveState.get(key.toString())).add(myInst);

            // NOW, FOR THE TAPES
            for (int k = 0; k < nbTapes; k++) {
                if (p.getInstComp(i, ((4 * k) + 3)).equals("*")) // si '*' au char crit, on avait forcement * au lu...=> on ecrit le meme que le lu
                 {
                    myInst.setTrans(k,
                        (String) (key.toString().split("[.]")[(2 * k) + 2] +
                        "." + p.getInstComp(i, (4 * k) + 4)));
                } else {
                    myInst.setTrans(k,
                        (String) (p.getInstComp(i, (4 * k) + 3) + "." +
                        p.getInstComp(i, (4 * k) + 4)));

                    // this long lines copy key and as value the char to write and the shift (at 4k+3 and +4)
                }
            }
        }
        /* ************ END ND *************** */
        else { // case of several tapes, no undetermination
            myInst = new Instruction(p.getLine(i),
                    (String) p.getInstComp(i, (4 * (nbTapes)) + 1));
            moveState.put(key.toString(), myInst);

            // 4*nbTapes+1 = indice of last component = newState
            // for each tape, copy it transitions...
            for (int k = 0; k < nbTapes; k++) {
                if (p.getInstComp(i, ((4 * k) + 3)).equals("*")) // si '*' au char crit, on avait forcement * au lu...=> on ecrit le meme que le lu
                 {
                    myInst.setTrans(k,
                        (String) (key.toString().split("[.]")[(2 * k) + 2] +
                        "." + p.getInstComp(i, (4 * k) + 4)));
                } else {
                    myInst.setTrans(k,
                        (String) (p.getInstComp(i, (4 * k) + 3) + "." +
                        p.getInstComp(i, (4 * k) + 4)));

                    // this long lines copy key and as value the char to write and the shift (at 4k+3 and +4)
                }
            }
        }
    }

    ////////////////////////////////////////////////////////////////////

    /**
     * generate the key (curstate + ( tapeNb + charRead)for each tape ) from an instruction (its number)
     *
     * @param i the number of instruction line concerned (to search in TmParser.ordInst ...)
     * @param key la cl  crer
     *
     * */
    private String createKey(int i) {
        int tapes = nbTapes;
        StringBuffer key = new StringBuffer();
        int starPos;
        key.append(p.getInstComp(i, 0)); // insert the current state info
        key.append("."); // separator

        for (int j = 0; j < (tapes - 1); j++) // for each tape sequence select the char read
         {
            key.append(Integer.toString(j)); // on ne fait pas la dernire bande..voir plus bas
            key.append(".");

            key.append(p.getInstComp(i, (4 * j) + 2)); // 'cos 4*i+2 = char read by tape no i

            // }
            key.append(".");
        }

        if (nbTapes != 1) {
            key.append(Integer.toString(tapes - 1)); // the last tape number
        } // ONE TAPE:
        else {
            key.append("0"); // le numro de la bande
        }

        key.append(".");

        if (nbTapes == 1) {
            key.append(p.getInstComp(i, 1));
        } else {
            key.append(p.getInstComp(i, (4 * (tapes - 1)) + 2)); // append le char read par la dernire tape (isole car pas de "." after)
        }

        if ((starPos = key.toString().indexOf('*')) != -1) {
            /*
            !!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!
            !!!!!!!!!!!!!!!!!!!!!!
            */
            genericStar(key.toString(),
                Integer.parseInt(key.toString().substring(starPos - 2,
                        starPos - 1)), starPos, i);

            return "np";
        }

        // (else)
        return key.toString();
    }

    ////////////////////////////////////////////////////////////////////

    /**
    * there we've found a '*'
    * @param tapeConcerned is the tape number of the tape which has to be its '*' replaced
    * @param nbInst the number of the instruction we work on
    * @param key: the key created
    * @param place the index of the star we'll make disappear
    */

    /*

    si l'alphabet de la tape est vide on utilisera celui de la machine

    */
    private void genericStar(String key, int tapeConcerned, int place,
        int nbInst) {
        String buf = null;
        Alphabet alphaB;
        int starPos;

        if (((Tape) rubans.get(tapeConcerned)).getAlphaSize() == 0) {
            alphaB = ((Tape) rubans.get(tapeConcerned)).getAlpha();
        } else {
            alphaB = alpha;
        }

        // loop on every letter
        for (int i = 0; i < alpha.size(); i++) {
            // replace the char targetted
            buf = replace(key, (Character) alpha.get(i), place);

            // if another '*' is in our key..
            if ((starPos = buf.indexOf('*')) != -1) {
                genericStar(buf,
                    Integer.parseInt(key.substring(starPos - 2, starPos - 1)),
                    buf.indexOf('*'), nbInst);
            } else if (nbTapes == 1) {
                insertOneTapeTransitions(buf.toString(), nbInst);
            } else {
                insertSeveralTapesTransitions(buf.toString(), nbInst);
            }

            buf = null;
        }
    }

    ////////////////////////////////////////////////////////////////////
    private void genericStarVariable(String key, int tapeConcerned, int place,
        int nbInst) {
        String buf = null;
        Alphabet alphaB;
        int starPos;

        if (((Tape) rubans.get(tapeConcerned)).getAlphaSize() == 0) {
            alphaB = ((Tape) rubans.get(tapeConcerned)).getAlpha();
        } else {
            alphaB = alpha;
        }

        // loop on every letter
        for (int i = 0; i < alpha.size(); i++) {
            // replace the char targetted
            buf = replace(key, (Character) alpha.get(i), place);

            // if another '*' is in our key..
            if ((starPos = buf.indexOf('*')) != -1) {
                genericStarVariable(buf,
                    Integer.parseInt(key.substring(starPos - 2, starPos - 1)),
                    buf.indexOf('*'), nbInst);
            } else {
                int tapesCard = (int) ((p.getInstCard(nbInst) - 2) / 4);

                insertVariableTransitions(buf.toString(), nbInst, tapesCard);
            }

            buf = null;
        }
    }

    ////////////////////////////////////////////////////////////////////

    /**
    *
    *
    * @param word the word in which we'll replace a symbol
    * @param letter the new letter
    * @param place its index in the word
    *
    */
    private String replace(String word, Character letter, int place) {
        StringBuffer buf = new StringBuffer(word.substring(0, place));
        buf.append(letter);
        buf.append(word.substring(place + 1));

        return buf.toString();
    }

    ////////////////////////////////////////////////////////////////////

    /**
       * generate the key (curstate + ( tapeNb + charRead)for each tape ) from an instruction (its number)
       *
       * @param i the number of instruction line concerned (to search in TmParser.ordInst ...)
       * @param key la cl  crer
       * @param tapes number of tapes concerned.
       * */
    private String createKeyUnsized(int i, int tapes) {
        StringBuffer key = new StringBuffer();
        key.append(p.getInstComp(i, 0)); // insert the current state info
        key.append("."); // separator

        for (int j = 0; j < (tapes - 1); j++) // for each tape sequence select the char read
         {
            key.append(p.getInstComp(i, (4 * j) + 1)); // number of the tape concerned
            key.append(".");
            key.append(p.getInstComp(i, (4 * j) + 2)); // 'cos 4*i+2 = char read by tape no i
            key.append(".");
        }

        key.append(p.getInstComp(i, (4 * (tapes - 1)) + 1));
        key.append(".");
        key.append(p.getInstComp(i, (4 * (tapes - 1)) + 2)); // append le char read par la dernire tape (isole car pas de "." after)

        int starPos;

        if ((starPos = key.toString().indexOf('*')) != -1) {
            genericStarVariable(key.toString(),
                Integer.parseInt(key.toString().substring(starPos - 2,
                        starPos - 1)), starPos, i);

            return "np";
        } else {
            return key.toString();
        }
    }

    ////////////////////////////////////////////////////////////////////

    /**
     * s'occupe des intructions de taille varialbe..
     *
     * */
    private void specialSizedInst(int i) {
        String key;

        /////////// On suppose - pour l'instant ? - qu'on a pas de charas gnriques dans notre instruction
        /////// on detecte les bandes concernes, on les mets dans un vecteur
        int tapesCard = (int) ((p.getInstCard(i) - 2) / 4); // nombre de bandes concernes, on a 4*tapesCard + 2 ( 2 pour etat init et final)

        ////// on cre la cl avec createKey en passant le nbre de bandes <=> la taille de l'inst
        key = createKeyUnsized(i, tapesCard);

        if (!key.equals("np")) {
            insertVariableTransitions(key, i, tapesCard);
        }
    }

    ////////////////////////////////////////////////////////////////////

    /**
    * insert the variable transitions into the machine mappings
    * @param i instruction's index in TmParser structures
    */
    private void insertVariableTransitions(String key, int i, int tapesCard) {
        int nbt;
        Instruction myInst = new Instruction(p.getLine(i),
                ((String) p.getInstComp(i, (4 * (tapesCard)) + 1)));

        // { la cl dans key }
        ////// on met tout a dans moveStateUnsized,
        moveStateUnsized.put(key.toString(), myInst); // 4*nbTapes+1 = indice of last component = newState

        unsizedKeys.add(key.toString());

        ////// et dans transUnsized pour les bandes concernes seulement (les autres n'ont pas d'action  effectuer..)
        for (int n = 0; n < tapesCard; n++) // pour toute tape de myTapes
         {
            /*
            on doit insrer pour le numro de bande concerne !
            */
            nbt = Integer.parseInt(p.getInstComp(i, (4 * n) + 1));

            // if that tape has a '*' as char to write, it had '*' as read, now replaced
            if (p.getInstComp(i, (4 * n) + 3).equals("*")) {
                myInst.setTrans(nbt,
                    (String) (key.split("[.]")[(2 * n) + 2] + "." +
                    p.getInstComp(i, (4 * n) + 4)));
            } else {
                myInst.setTrans(nbt,
                    (String) (p.getInstComp(i, (4 * n) + 3) + "." +
                    p.getInstComp(i, (4 * n) + 4)));
            }
        }
    }

    ////////////////////////////////////////////////////////////////////

    /** true if the instruction i describes all tapes status...
     * ou  supprimer et utiliser simplement une condition sur le nombre d'lments.. -> 4*nbTapes+2
     * */
    private boolean fullSized(int i) {
        return (p.getInstCard(i) == ((4 * nbTapes) + 2));
    }

    ////////////////////////////////////////////////////////////////////

    /**
     * tests if entry word is formed with alphabet's letters
     * */
    private boolean wordCorrect(String s) {
        int ok = 0;

        for (int i = 0; i < s.length(); i++) {
            ok = 0;

            for (int j = 0; j < alpha.size(); j++) // NBEL DANS VECTOR ?


                if ((alpha.getLetter(j)).charValue() == s.charAt(i)) {
                    ok++; // =1 at end of loop if char belongs to alphabet
                }

            if (ok != 1) {
                return false;
            }
        }

        return true;
    }

    ////////////////////////////////////////////////////////////////////

    /**
    * @return true if s (a character) is in the alphabet // later: manage tape's own alphabets
    * */
    public boolean inAlphabet(char c) {
        if (c != '*') {
            return alpha.contains(new Character(c));
        }

        return true;
    }

    ////////////////////////////////////////////////////////////////////
    ////// SETTERS
    // mostly called by TmParser during the parsing of the file
    public void setStateStatus(String key, String value) {
        stateStatus.put(key, value);
    }

    public void setNbTapes(int i) {
        nbTapes = i;
    }

    public void setInWord(String s) {
        inWord = s;
    }

    public void setAlpha(int i, char c) {
        alpha.addLetter(i, c);
    }

    public void setSpeed(int i) {
        speed = i;
    }

    public void setName(String s) {
        name = new String(s);
    }

    public void setInitState(String s) {
        curState = s;
        initState = s;
    }

    public void setChoiceMachine() {
        choiceMachine = true;
    }

    public void setProbMachine() {
        setChoiceMachine();
        probMachine = true;
    }

    public void setGravity() {
        setProbMachine();
        gMachine = true;
    }
}
