/*
 * Copyright 2005 Jean-Philippe Aumasson <jpnossa@free.fr>
 * Copyright 2005 Florent Kaisser <jpnossa@free.fr>
 * Copyright 2005 Jean-Baptiste Langlois <florent.kaisser@free.fr>
 *
 * This file is part of SIMS : SIMS Is a Machine Simulator
 *
 * SIMS is free software; you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation; either version 2 of the License, or
 * (at your option) any later version.
 *
 * SIMS is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with SIMS; if not, write to the Free Software
 * Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
 *
 * For more details see the file COPYING.
 */
/**
* A Fifo structure containing IDs (MConfig), used for D-simulation of ND-machine
* @see /sims/log/algos for detailed algorithm description
*  @author JP <jpnossa@free.fr>
*  @version 20/12/04
*/
package sims.mdt.nd;

import sims.mdt.*;

import java.util.*;


public class Fifo {
    private LinkedList fifo; // the virtual fifo list itself

    ////////////////////////////////////////////////////////////////////
    public Fifo() {
        fifo = new LinkedList();
    }

    ////////////////////////////////////////////////////////////////////
    public Fifo(MConfig c) {
        this();
        fifo.addLast(c);
    }

    ////////////////////////////////////////////////////////////////////

    /**
    * Add an element at the bottom of the FIFO
    * @param c the <tt>MConfig</tt> to add to the FIFO.
    */
    public void add(MConfig c) {
        fifo.addLast(c);
    }

    ////////////////////////////////////////////////////////////////////

    /**
    * @return <tt>true</tt> if is the FIFO is empty (contains no MConfig (anymore)).
    */
    public boolean isEmpty() {
        return fifo.isEmpty();
    }

    ////////////////////////////////////////////////////////////////////
    ////// GETTERS

    /**
    * @return the top of the FIFO, and delete it, shifting all elements consequently.
    *
    */
    public MConfig getFirst() {
        return (MConfig) fifo.removeFirst();
    }

    ////////////////////////////////////////////////////////////////////

    /**
    * @return the number of MConfigs in the Fifo.
    */
    public int getSize() {
        return fifo.size();
    }
}
