/*
 * Copyright 2005 Jean-Philippe Aumasson <jpnossa@free.fr>
 * Copyright 2005 Florent Kaisser <jpnossa@free.fr>
 * Copyright 2005 Jean-Baptiste Langlois <florent.kaisser@free.fr>
 *
 * This file is part of SIMS : SIMS Is a Machine Simulator
 *
 * SIMS is free software; you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation; either version 2 of the License, or
 * (at your option) any later version.
 *
 * SIMS is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with SIMS; if not, write to the Free Software
 * Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
 *
 * For more details see the file COPYING.
 */
/**
* A instant m-configuration of a nondeterministic machine
* m-config <=> content of tapes (finite nb of chars) (Tape instances)
*               for each tape its scanner position (in Tape...)
*               current state (String)
*  @author JP <jpnossa@free.fr>
*  @version 20/12/04
*/
package sims.mdt.nd;

import sims.mdt.*;

import java.util.*;


public class MConfig {
    private Vector tapes; // the tapes off the config (fixed nb for a given TM)
    private String currentState; // the state of the m-config
    private int time; // time complexity for a deterministic-simulation

    ////////////////////////////////////////////////////////////////////

    /**
    * Initialize the <tt>MConfig</tt>.
    * @param bandes an array containing the tapes of the configuration.
    * @param etat the current state of the machine.
    * @param t the current time complexity value.
    */
    public MConfig(Tape[] bandes, String etat, int t) {
        tapes = new Vector();

        try {
            for (int i = 0; i < bandes.length; i++) {
                tapes.add((Tape) bandes[i].clone());
            }
        } catch (CloneNotSupportedException e) {
        }

        ;

        // toString() here used to avoid the reference 
        currentState = new String(etat.toString());
        time = t;
    }

    ////////////////////////////////////////////////////////////////////
    ////// GETTERS

    /**
    * @return the i-th tape stored, cloned
    */
    public Tape getTape(int i) {
        try {
            return (Tape) ((Tape) tapes.get(i)).clone();
        } catch (CloneNotSupportedException e) {
        }

        ;

        return null;
    }

    ////////////////////////////////////////////////////////////////////

    /**
    * @return the currentState String.
    */
    public String getState() {
        return currentState;
    }

    ////////////////////////////////////////////////////////////////////

    /**
    * @return the currentState String.
    */
    public int getTime() {
        return time;
    }
}
